﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using MARX.Security;

namespace WindowsFormsSample
{
    public partial class Form1 : Form
    {
        bool decimalUsed = false;
        IMarxProtection protection;
        const string error = "ERR";

        enum Operation
        {
            Add,
            Sub,
            Mul,
            Div,
            None
        }
        Operation previousOperation = Operation.None;
        Operation currentOperation = Operation.None;



        public Form1()
        {
            InitializeComponent();
            //protection = new ProtectionNetwork(101, true);
            //protection = new ProtectionLicenseFile("license.xml");
            protection = new ProtectionLocal(101);
            protection.validationSucceed += OnValidationSucceed;
            protection.validationFailed += OnValidationFailed;
        }

        private void Calculator_Load(object sender, EventArgs e)
        {
            protection.Start();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            decimalUsed = false;
            PreCheck_ButtonClick();
            if (tbDisplay.Text.Length > 0)
            {
                double d;
                if (!double.TryParse(tbDisplay.Text[tbDisplay.Text.Length - 1].ToString(), out d))
                {
                    previousOperation = Operation.None;
                }

                tbDisplay.Text = tbDisplay.Text.Remove(tbDisplay.Text.Length - 1, 1);
            }
            if (tbDisplay.Text.Length == 0)
            {
                previousOperation = Operation.None;
            }
            if (previousOperation != Operation.None)
            {
                currentOperation = previousOperation;
            }
        }

        private void btnClearAll_Click(object sender, EventArgs e)
        {
            decimalUsed = false;
            PreCheck_ButtonClick();
            previousOperation = Operation.None;
            tbDisplay.Clear();
        }
        private void PreCheck_ButtonClick()
        {
            if (tbDisplay.Text == error)
                tbDisplay.Clear();
            if (previousOperation != Operation.None)
            {
                EnableOperatorButtons();
            }
        }

        private void EnableOperatorButtons(bool enable = true)
        {
            btnMul.Enabled = enable;
            btnDiv.Enabled = enable;
            btnAdd.Enabled = enable;
            if (!enable)
            {
                decimalUsed = false;
            }
            //btnSub.Enabled = enable;
        }

        public void OnValidationSucceed(object sender, ProtectionMessage message)
        {
            this.Invoke(new Action(() =>
            {
                Thread.Sleep(100);
                this.Enabled = true;
            }));
        }

        public void OnValidationFailed(object sender, ProtectionMessage message)
        {
            this.Invoke(new Action(() =>
            {
                Thread.Sleep(100);
                this.Enabled = false;
            }));

            Task _task = new Task(() =>
            {
                Thread.Sleep(1000);
                this.Invoke(new Action(() => {
                    MessageBox.Show(message.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }));
            });
            _task.Start();
        }

        private void btnDiv_Click(object sender, EventArgs e)
        {
            if (tbDisplay.TextLength == 0) return;
            PreCheck_ButtonClick();
            currentOperation = Operation.Div;
            PerformCalculation(previousOperation);

            previousOperation = currentOperation;
            EnableOperatorButtons(false);
            tbDisplay.Text += (sender as Button).Text;
        }

        private void btnMul_Click(object sender, EventArgs e)
        {
            if (tbDisplay.TextLength == 0) return;
            PreCheck_ButtonClick();
            currentOperation = Operation.Mul;
            PerformCalculation(previousOperation);

            previousOperation = currentOperation;
            EnableOperatorButtons(false);
            tbDisplay.Text += (sender as Button).Text;
        }

        private void PerformCalculation(Operation previousOperation)
        {
            try
            {
                if (previousOperation == Operation.None)
                    return;
                List<double> lstNums = null;

                switch (previousOperation)
                {
                    case Operation.Add:
                        if (currentOperation == Operation.Sub)
                        {
                            currentOperation = Operation.Add;
                            return;
                        }
                        lstNums = tbDisplay.Text.Split('+').Select(double.Parse).ToList();
                        tbDisplay.Text = (lstNums[0] + lstNums[1]).ToString();
                        break;
                    case Operation.Sub:
                        int idx = tbDisplay.Text.LastIndexOf('-'); // To handle ex: -9-2
                        if (idx > 0)
                        {
                            var op1 = Convert.ToDouble(tbDisplay.Text.Substring(0, idx));
                            var op2 = Convert.ToDouble(tbDisplay.Text.Substring(idx + 1));
                            tbDisplay.Text = (op1 - op2).ToString();
                        }
                        break;
                    case Operation.Mul:
                        if (currentOperation == Operation.Sub)
                        {
                            currentOperation = Operation.Mul;
                            return;
                        }
                        lstNums = tbDisplay.Text.Split('*').Select(double.Parse).ToList();
                        tbDisplay.Text = (lstNums[0] * lstNums[1]).ToString();
                        break;
                    case Operation.Div:
                        if (currentOperation == Operation.Sub)
                        {
                            currentOperation = Operation.Div;
                            return;
                        }
                        try
                        {
                            lstNums = tbDisplay.Text.Split('/').Select(double.Parse).ToList();
                            if (lstNums[1] == 0)
                            {
                                throw new DivideByZeroException();
                            }
                            tbDisplay.Text = (lstNums[0] / lstNums[1]).ToString();
                        }
                        catch (DivideByZeroException)
                        {
                            tbDisplay.Text = error;
                        }
                        break;
                    case Operation.None:
                    default:
                        break;
                }
            }
            catch (Exception)
            {
                tbDisplay.Text = error;
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (tbDisplay.TextLength == 0) return;
            PreCheck_ButtonClick();
            currentOperation = Operation.Add;
            PerformCalculation(previousOperation);

            previousOperation = currentOperation;
            EnableOperatorButtons(false);
            tbDisplay.Text += (sender as Button).Text;
        }

        private void btnSub_Click(object sender, EventArgs e)
        {
            if (tbDisplay.TextLength == 0 || previousOperation == Operation.Sub) return;
            PreCheck_ButtonClick();
            currentOperation = Operation.Sub;
            PerformCalculation(previousOperation);

            previousOperation = currentOperation;
            EnableOperatorButtons(false);
            tbDisplay.Text += (sender as Button).Text;
        }

        private void btnResult_Click(object sender, EventArgs e)
        {
            if (tbDisplay.TextLength == 0) return;
            if (previousOperation != Operation.None)
                PerformCalculation(previousOperation);

            previousOperation = Operation.None;

        }

        private void btnDecimal_Click(object sender, EventArgs e)
        {
            if (decimalUsed) return;
            if (tbDisplay.Text == error)
            {
                tbDisplay.Text = string.Empty;
            }
            EnableOperatorButtons();
            PreCheck_ButtonClick();
            tbDisplay.Text += (sender as Button).Text;
            decimalUsed = true;
        }

        private void btnNum_Click(object btn, EventArgs e)
        {
            if (tbDisplay.Text == error)
            {
                tbDisplay.Text = string.Empty;
            }
            EnableOperatorButtons();
            PreCheck_ButtonClick();
            tbDisplay.Text += (btn as Button).Text;
        }
    }
}
