#include "cdo_helpers.h"
#include "stdio.h"
#include "smarx_utils.h"

// set here RSA key specific for your CBU
BYTE* distrRSAKeyModulus = CBIOS_DEMO_DISTR_RSA_KEY_MODULUS;
BYTE* distrRSAKeyExponent = CBIOS_DEMO_DISTR_RSA_KEY_EXPONENT;

DWORD distr_cdo_create_signed_data(BYTE* data, DWORD dataSize, DWORD dwBoxName,
	CDO_SIGNED_DATA** signedData, DWORD* signDataSize)
{
	DO_RSA_EX_DATA refData = { DO_RSA_EX_KEY_INDEX_NONE };
	CBIOS_RSA_KEY rsaPrivKey = { sizeof rsaPrivKey };
	const DWORD len = CDO_SIGNED_DATA_SIZE(dataSize);
	struct CDO_SIGNED_DATA* pSignedData;
	DWORD dwRet;
#ifndef __BORLANDC__
	DO_KEY_REF rsaDistr = { KEY_REF_KEY_TYPE_EXTNL, {0}, &refData, &rsaPrivKey };
#else
	DO_KEY_REF rsaDistr = { KEY_REF_KEY_TYPE_EXTNL };
	rsaDistr.keyDescriptor = &refData;
	rsaDistr.keyRSA = &rsaPrivKey;
#endif

	refData.dwBoxName = dwBoxName;

	*signedData = NULL;
	*signDataSize = 0;
	pSignedData = (CDO_SIGNED_DATA*)malloc(len);
	ZeroMemory((BYTE*)pSignedData, sizeof * pSignedData);

	rsaPrivKey.wModulusLen = rsaPrivKey.wPrivExpLen = CBIOS_INTERNAL_RSA_LEN;
	memcpy(rsaPrivKey.ubModulus, distrRSAKeyModulus, CBIOS_INTERNAL_RSA_LEN);
	memcpy(rsaPrivKey.ubPrivExp, distrRSAKeyExponent, CBIOS_INTERNAL_RSA_LEN);
	pSignedData->dwSignSize = sizeof pSignedData->signature;
	dwRet = TEOS_CalculateDigitalSignature(NULL, &rsaDistr, TEOSDO_SIGN_ALG_TYPE_BOXNAME, 0, data, dataSize, pSignedData->signature, &pSignedData->dwSignSize, NULL);
	if (dwRet)
	{
		free(pSignedData);
		return dwRet;
	}
	*signedData = pSignedData;
	*signDataSize = len;
	pSignedData->dwDataSize = dataSize;
	memcpy(pSignedData->data, data, dataSize);

	return 0;
}

DWORD client_get_boxname(DWORD* boxName)
{
	CBIOS_BOX_INFO boxInfo;
	DWORD dwRet = CBIOS_GetBoxInfo(&boxInfo);
	if (dwRet) return dwRet;
	*boxName = boxInfo.dwBoxName;
	return 0;
}
DWORD client_cdo_op(DWORD dwDoID, DWORD dwOperation, CDO_SIGNED_DATA* signedData, DWORD signDataSize, DWORD dwParameter, PASSW bPass)
{
	DWORD dwRet;
	BYTE* pData = (BYTE*)signedData;
	DWORD len = signDataSize;

	dwRet = TEOS_DoOperation(dwDoID, dwOperation, pData, len, dwParameter, bPass);

	return dwRet;
}

// -----------------------------------------------------------------------
DWORD _cdo_op(DWORD dwDoID, DWORD dwOperation, BYTE* pData, DWORD dwDataSize, DWORD dwParameter, PASSW bPass, DWORD* pdwExpected)
{
	struct CDO_SIGNED_DATA* signedData = NULL;
	DWORD boxName, signedDataSize, dwRet;

	do
	{
		dwRet = client_get_boxname(&boxName);//or get from DB
		if (dwRet) break;

		dwRet = distr_cdo_create_signed_data(pData, dwDataSize, boxName, &signedData, &signedDataSize);
		if (dwRet) break;

		dwRet = client_cdo_op(dwDoID, dwOperation, signedData, signedDataSize, dwParameter, bPass);
		free(signedData);
	} while (0);
	PrintExpectedErrorEx(dwRet, pdwExpected);
	return dwRet;
}

// -----------------------------------------------------------------------
DWORD cdo_set(DWORD dwDoID, BYTE* pData, DWORD dwDataSize)
{
	// SET DATA OBJECT
	return _cdo_op(dwDoID, TEOSDO_SET, pData, dwDataSize, 0, NULL, NULL);
}
DWORD cdo_set_error_checking(DWORD dwDoID, BYTE* pData, DWORD dwDataSize, DWORD dwExpected)
{
	// SET DATA OBJECT
	return _cdo_op(dwDoID, TEOSDO_SET, pData, dwDataSize, 0, NULL, &dwExpected);
}
// -----------------------------------------------------------------------
DWORD cdo_def_unlim_ex(DWORD dwDoID)
{
	// UNLIMITED DATA OBJECT
	printf(":: TEOS_DoUnlimitedEx\t");
	return _cdo_op(dwDoID, TEOSDO_UNLIMITED_EX, NULL, 0, 0, NULL, NULL);
}
DWORD cdo_inc(DWORD dwDoID, BYTE* pData, DWORD dwDataSize)
{
	// SET DATA OBJECT
	return _cdo_op(dwDoID, TEOSDO_INC, pData, dwDataSize, 0, NULL, NULL);
}
DWORD cdo_def_inc(DWORD dwDoID, const char* szUnit)
{
	// INCREMENT DATA OBJECT
	printf(":: TEOS_DoInc (1 %s)\t", szUnit);
	return _cdo_op(dwDoID, TEOSDO_INC, NULL, 0, 0, NULL, NULL);
}


// -----------------------------------------------------------------------
//		Auxiliary functions for DO:
// -----------------------------------------------------------------------
DWORD _do_def_dec(DWORD dwDoID, CHAR* szUnit, DWORD* pdwExpected)
{
	DWORD dwRet;
	// DECREMENT DATA OBJECT
	printf(":: TEOS_DoDec (1 %s)", szUnit);
	if (strlen(szUnit) < 4) printf("\t");
	dwRet = TEOS_DoDec(dwDoID, NULL, 0, 0, NULL);
	PrintExpectedErrorEx(dwRet, pdwExpected);
	return dwRet;
}
DWORD do_def_dec(DWORD dwDoID, CHAR* szUnit)
{
	return _do_def_dec(dwDoID, szUnit, NULL);
}
DWORD do_def_dec_error_checking(DWORD dwDoID, CHAR* szUnit, DWORD dwExpected)
{
	return _do_def_dec(dwDoID, szUnit, &dwExpected);
}
// -----------------------------------------------------------------------
DWORD do_def_clear(DWORD dwDoID)
{
	DWORD dwRet;
	// CLEAR DATA OBJECT
	printf(":: TEOS_DoClear\t\t");
	dwRet = TEOS_DoClear(dwDoID, NULL);
	PrintErrorCode(dwRet);
	return dwRet;
}
// -----------------------------------------------------------------------
DWORD _do_verify(DWORD dwDoID, DWORD* pdwData, DWORD* pdwExpected)
{
	DWORD dwRet, dwDataSize = pdwData ? sizeof(DWORD) : 0;
	// VERIFY
	printf(":: TEOS_DoVerify");
	if (pdwData) printf(" (by %i)", *pdwData);
	else printf("\t");
	dwRet = TEOS_DoVerify(dwDoID, (BYTE*)pdwData, dwDataSize, 0, NULL);
	PrintExpectedErrorEx(dwRet, pdwExpected);
	return dwRet;
}
DWORD _do_def_verify(DWORD dwDoID, DWORD* pdwExpected)
{
	return _do_verify(dwDoID, NULL, pdwExpected);
}
DWORD do_verify(DWORD dwDoID, DWORD dwData)
{
	return _do_verify(dwDoID, &dwData, NULL);
}
DWORD do_verify_error_checking(DWORD dwDoID, DWORD dwData, DWORD dwExpected)
{
	return _do_verify(dwDoID, &dwData, &dwExpected);
}
DWORD do_def_verify(DWORD dwDoID)
{
	return _do_def_verify(dwDoID, NULL);
}
DWORD do_def_verify_error_checking(DWORD dwDoID, DWORD dwExpected)
{
	return _do_def_verify(dwDoID, &dwExpected);
}
