/*****************************************************************************
  Copyright:   : Copyright (c) 2002, 2016 by MARX(R) CryptoTech LP
  Project      : CBIOS: CRYPTO-BOX Input/Output System API
  Tools        : MSVC++ 6.0 - 2015, C++ Builder 5 - XE6
  Description  : CBIOS API definitions
  Platform     : Platform independent
******************************************************************************
  VER         DATE        WHO   DETAIL                        
******************************************************************************
  1.00        25Feb2003   DI    Initial version (MPCB)
  1.10        31May2003   VS    CBIOS project started
  1.10        18Jun2003   VS    
  1.20        21Mar2005   VS    CT-M2048 limited support added
  1.30        29Aug2005   VS    CBIOS_GetDeveloperID
  1.50        28Jan2009   AL    CBU2 Support
  1.51        18Mar2010   VG    Network notifications added
  1.60        19May2010   VG    CBIOS_IsBoxLockedByOthers[I] added
  1.70        17Oct2011   VG    added network extended licensing
  1.71        21May2013   VG    CBIOS_ClientLogin added
  1.72        11Sep2014   VG    CBIOS_ScanNetworkEx added
  1.80        05Jan2016   VG    CBIOS_CBM_GetVolumePath added, CBUSD support
  1.81        15Jun2016   VG    CBIOS_ConnectEx added, CBIOS_ScanNetworkEx fixed
  1.81        17Jul2019   AM    Universal CBIOS header for Win, Mac and Linux
  1.81        23Jul2020   AR    IPv6 support
  1.81        12Nov2020   AM    Added Set UPW/APW commnad for network server
  1.81        27Jul2021   AM    Add RSA error codes
*****************************************************************************/

#ifndef _CBIOS_H_
#define _CBIOS_H_

#ifdef WIN32
#define PACKED
#else
#define PACKED __attribute__((packed))
#endif


#ifdef WIN32
#ifndef CBIOS_NOFORCE_LIBS
#ifdef _MSC_VER
#pragma comment(lib, "ws2_32")
#pragma comment(lib, "shlwapi")
#ifdef _WIN64
#define CBIOS_LINK_LIB_NAME "cbios64"
#else
#define CBIOS_LINK_LIB_NAME "cbios"
#endif
#elif defined(__BORLANDC__)
#ifdef _WIN64
#pragma comment(lib, "ws2_32")
#pragma comment(lib, "shlwapi")
#define CBIOS_LINK_LIB_NAME "cbios_bc"
#else
#pragma comment(lib, "ws2_32.lib")
#pragma comment(lib, "shlwapi.lib")
#define CBIOS_LINK_LIB_NAME "cbios_bc.lib"
#endif//_WIN64
#endif//_MSC_VER
#ifdef CBIOS_LINK_LIB_NAME
#pragma comment(lib, CBIOS_LINK_LIB_NAME)
#endif//CBIOS_LINK_LIB_NAME
#endif//CBIOS_NOFORCE_LIBS
#endif//WIN32

#define CBIOS_MAX_BOXES       32
#define CBIOS_MAX_APPS        32

#define CBIOS_BOX_UNKNOWN	(-1)
#define CBIOS_BOX_CBU		0
#define CBIOS_BOX_CBU2		1

#define CBIOS_APPID_SYSTEM         0
#define CBIOS_APPID_SPO            1
#define CBIOS_APPID_XENODISC       2
#define CBIOS_APPID_GINA           3
#define CBIOS_APPID_AUTOPROTECTION 4
#define CBIOS_APPID_LMS            5
#define CBIOS_APPID_PDFPROTECTION  6                            
#define CBIOS_APPID_CSP            7
#define CBIOS_APPID_PKCS11         8
#define CBIOS_APPID_PKCS11_LIGHT   9
#define CBIOS_APPID_PRJ_EDITIONS  10
//#define CBIOS_APPID_FLMS          11

#define CBIOS_LABEL_LEN 8

#define CBIOS_WARNING_MASK             0x80        // nonfatal error codes mask
//CBU error codes
#define CBIOS_SUCCESS                  0x00        // no error
#define CBIOS_ERR_NOT_FOUND            0x01        // No box with BoxName
#define CBIOS_ERR_NOT_LOGGED           0x02        // Missing UPW login
#define CBIOS_ERR_WRONG_PARAM          0x03        // Bad input parameter
#define CBIOS_ERR_RAM1_NOT_EXIST       0x04        // Range of RAM1 incorect
#define CBIOS_ERR_RAM2_NOT_EXIST       0x05        // ...
#define CBIOS_ERR_RAM3_NOT_EXIST       0x06        // ...
#define CBIOS_ERR_BOX_IS_LOCKED        0x07        // Box used another application
#define CBIOS_ERR_EXT_ERROR            0x08        // see SM_Last_Error
#define CBIOS_ERR_NO_MEMORY            0x09        // allocation error
#define CBIOS_CBUSB_ERR_NOT_IMPLEMENTED 0x0A        // not implemented by driver or firmware

// CBU2 specific error codes
#define CBIOS_CBU2_ERR_CONFIG_LOCKED      0x1010      // config locked
#define CBIOS_CBU2_ERR_LOAD_FIRMWARE      0x1020      // firmware upload error
#define CBIOS_CBU2_ERR_RAM4_NOT_EXIST     0x1030      // no RSA keys
#define CBIOS_CBU2_ERR_RAM5_NOT_EXIST     0x1031      // no AES keys
#define CBIOS_CBU2_ERR_KEY_LOCKED         0x1032      // key is locked
#define CBIOS_CBU2_ERR_KEY_NOTINITIALIZED 0x1033      // key is not used
#define CBIOS_CBU2_ERR_CRYPTO_FAILED      0x1034      // crypto operation failed

//CBIOS error codes
#define CBIOS_ERR_BOX_NOT_OPENED       0x21        // attempt to access the box, 
                                                   //  which wasn't opened with CBIOS_OpenBox call
#define CBIOS_ERR_BOX_ALREADY_OPENED   0x22        // attempt to open box repeatedly
//#define CBIOS_ERR_NO_RAM               0x23        // (obsolete) memory allocation error (computer, not box RAM)
#define CBIOS_ERR_TOO_MANY_BOXES_OPEN  0x23        // (!has same code as CBIOS_ERR_NO_RAM!) opened boxes table is full
#define CBIOS_ERR_BOX_NOT_FOUND        0x24
#define CBIOS_ERR_APP_NOT_FOUND        0x25
#define CBIOS_ERR_BOX_NOT_SUPPORTED    0x26        // not supported firmware
#define CBIOS_ERR_OUT_OF_RANGE         0x27        // invalid index, address or block size
#define CBIOS_ERR_BOX_NOT_FORMATTED    0x28        
#define CBIOS_ERR_CORRUPTED_CBIOS_DATA 0x29        
#define CBIOS_ERR_NO_SPACE_IN_RAM1     0x2A        
#define CBIOS_ERR_NO_SPACE_IN_RAM2     0x2B        
#define CBIOS_ERR_NO_SPACE_IN_RAM3     0x2C        
#define CBIOS_ERR_APP_ALREADY_EXISTS   0x2D
#define CBIOS_ERR_BOX_ALREADY_LOGGED   0x2E        
#define CBIOS_ERR_BOX_NOT_LOGGED       0x2F        
#define CBIOS_ERR_EXTENDED_MODE        0x30        // some application use extended mode
#define CBIOS_ERR_NOT_EXTENDED_MODE    0x31        // can't execute extended function in standart mode
#define CBIOS_ERR_LOCK_TIMEOUT         0x32        // box locked more then 20 sec
#define CBIOS_ERR_WRONG_SIGNATURE      0x33        // CRC error while decrypt box signature
#define CBIOS_ERR_WRONG_KEY            0x34        // wrong internal RSA key
#define CBIOS_ERR_NOT_INITIALIZED      0x35        // CBIOS not initialized (CBIOS_Startup was not completed correctly)
#define CBIOS_ERR_NOT_LICENCED         0x36        // box not licenced for application
#define CBIOS_ERR_NO_SPACE             0x37        // no free space in box for operation
#define CBIOS_ERR_CRC_ERROR            0x38        
#define CBIOS_ERR_NO_FREE_LICENCE      0x39        // no free licence - can't open application
#define CBIOS_ERR_INVALID_HANDLE       0x3A
#define CBIOS_WRN_INIT_BOX_NOTIFIC    (0x3B | CBIOS_WARNING_MASK) // NONFATAL ERROR: box insert notifications subsystem is not initialized correctly,
                                                   //   but CBIOS run ok. Your need to call CBIOS_Finish if that error received from CBIOS_Startup
#define CBIOS_WRN_CTM_NOT_INITIALIZED (0x3C | CBIOS_WARNING_MASK) // NONFATAL ERROR: CTM2048 subsystem is not initialized correctly,
                                                   //   but CBIOS run ok. Your need to call CBIOS_Finish if that error received from CBIOS_Startup


#define CBIOS_ERR_NOT_IMPLEMENTED      0x40        //   not implented in CBIOS

#define CBIOS_ERR_CONN_REFUSED         0x60
#define CBIOS_ERR_WRONG_SRV_PASSWD     0x61
#define CBIOS_ERR_HANDSHAKE            0x62
#define CBIOS_ERR_NOT_CONNECTED        0x63
#define CBIOS_ERR_ALREADY_CONNECTED    0x64
#define CBIOS_ERR_TOO_MANY_CONNECTIONS 0x65
#define CBIOS_ERR_TIMEOUT              0x67
#define CBIOS_ERR_NOT_ALLOWED          0X68

#define CBIOS_ERR_INIT_IN_PROGRESS     0x66        // CBIOS initialization process is not yet complete

//CBIOS error codes for CBU2
#define CBIOS_CBU2_ERR_NO_SPACE_IN_RAM4     0x201
#define CBIOS_CBU2_ERR_NO_SPACE_IN_RAM5     0x202

//RSA error codes
#define RE_CONTENT_ENCODING 0x0400
#define RE_DATA 0x0401
#define RE_DIGEST_ALGORITHM 0x0402
#define RE_ENCODING 0x0403
#define RE_KEY 0x0404
#define RE_KEY_ENCODING 0x0405
#define RE_LEN 0x0406
#define RE_MODULUS_LEN 0x0407
#define RE_NEED_RANDOM 0x0408
#define RE_PRIVATE_KEY 0x0409
#define RE_PUBLIC_KEY 0x040a
#define RE_SIGNATURE 0x040b
#define RE_SIGNATURE_ENCODING 0x040c
#define RE_ENCRYPTION_ALGORITHM 0x040d
#define RE_MEM_ALLOC 0x09

// CBIOS_BOX_INFO.bLogin
#define CBIOS_UPW_LOGIN       0x01              // Flags from function SM_GetSecurityFlags
#define CBIOS_APW_LOGIN       0x02              // ...
#define CBIOS_ERROR_LOGIN     0x80              // ...

#define CBIOS_RAM1            0x01    
#define CBIOS_RAM2            0x02    
#define CBIOS_RAM3            0x03    

#define CBIOS_CT_FLAG           0x10
#define CBIOS_MEMORY_FLAG       0x40 // with flash memory
#define CBIOS_SD_FLAG           0x80 // as SD card

#define CBIOS_CB_MODEL_LEGACY   0x00 // old box
#define CBIOS_CB_MODEL_VERSA    0x01 // CRYPTO-BOX(r) VERSA
#define CBIOS_CB_MODEL_XS       0x02 // CRYPTO-BOX(r) XS
#define CBIOS_CB_MODEL_XL       0x03 // CRYPTO-BOX(r) XL
#define CBIOS_CT_MODEL_XS       0x12 // CrypToken(r) XS
#define CBIOS_CT_MODEL_XL       0x13 // CrypToken(r) XL
#define CBIOS_CT_MODEL_2048     0x20 // CT2048

// internal RSA
#define CBIOS_INTERNAL_RSA_BITS 1024
#define CBIOS_INTERNAL_RSA_LEN  (CBIOS_INTERNAL_RSA_BITS/8)
#define CBIOS_INTERNAL_RSA_ID_DISTR_PUBL    1
#define CBIOS_INTERNAL_RSA_ID_CLIENT_PRIV   2

// CBU2 (internal) RSA access flags
#define CBIOS_CBU2_ACCESS_NEVER     0
#define CBIOS_CBU2_ACCESS_ALWAYS    1
#define CBIOS_CBU2_ACCESS_UPW       2
#define CBIOS_CBU2_ACCESS_APW       3
#define CBIOS_CBU2_ACCESS_LOCK      4

// box signature
#define CBIOS_BOXSIGN_RSA_BITS  1024
#define CBIOS_BOXSIGN_RSA_LEN  (CBIOS_BOXSIGN_RSA_BITS/8)
#define CBIOS_BOXSIGN_RSA_PUBLIC_MODULUS {0xb5, 0xf5, 0x83, 0x2d, 0x13, 0x51, 0x67, 0x76,  0xc0, 0x94, 0x08, 0x31, 0x07, 0xdd, 0x65, 0x4c,  0x6c, 0xfc, 0x28, 0x0f, 0x1e, 0xfc, 0xfb, 0x0f,  0x86, 0x7d, 0x5d, 0x79, 0x01, 0x4e, 0x2b, 0xb6,  0xd8, 0xe6, 0xfc, 0x22, 0x53, 0xeb, 0x8d, 0x7d,  0xe4, 0x75, 0xc5, 0xa5, 0xf1, 0x5f, 0xd2, 0xb5,  0x07, 0xf6, 0x08, 0xa9, 0x73, 0x73, 0x05, 0x58,  0x82, 0xb0, 0xbd, 0x63, 0x10, 0xe4, 0x27, 0xd2,  0xb9, 0x2f, 0x5f, 0xc2, 0x29, 0xb3, 0xa6, 0x4c,  0x60, 0x43, 0xc2, 0xa9, 0x2b, 0xd3, 0x3e, 0xdd,  0xa1, 0xe7, 0xb2, 0x61, 0x38, 0xe5, 0x62, 0x12,  0xde, 0x62, 0xb9, 0xd2, 0x34, 0xf0, 0x71, 0xd6,  0x9a, 0x7d, 0xb7, 0x36, 0x3f, 0x73, 0x48, 0xc0,  0x33, 0xea, 0x1f, 0xbf, 0x28, 0x1d, 0xf1, 0x07,  0x0b, 0xa5, 0x41, 0x6c, 0x8c, 0x52, 0x26, 0x2d,  0x82, 0x31, 0x28, 0xf9, 0x3c, 0x0d, 0x0b, 0x5f};

#pragma pack(1)

typedef struct
{ 
  DWORD dwBoxName;
  DWORD dwModel;
  DWORD dwApp;          // application ID or applications qty
  WORD  wCBIOS;         // CBIOS version. 0 - non CBIOS box. 
  BYTE  bAppPolicy;     // Application Policy
  BYTE  bLogin;         // login status from cbusb_flag structure
  BYTE  bHiVersion;
  BYTE  bLoVersion;
  DWORD dwRAMLen;       // in bytes
  DWORD dwSize_RAM1;    // Size in RAM1
  DWORD dwSize_RAM2;    // Size in RAM2
  DWORD dwSize_RAM3;    // Size in RAM3
  BYTE  bBoxLabel[CBIOS_LABEL_LEN];      // Box label
} PACKED CBIOS_BOX_INFO;

typedef struct
{
  DWORD dwStructSize;
  DWORD dwBoxName;
  DWORD dwModel;
  DWORD dwApp;          // application ID or applications qty
  WORD  wCBIOS;         // CBIOS version. 0 - non CBIOS box. 
  BYTE  bAppPolicy;     // Application Policy
  BYTE  bLogin;         // login status from cbusb_flag structure
  BYTE  bHiVersion;
  BYTE  bLoVersion;
  DWORD dwRAMLen;       // in bytes
  DWORD dwSize_RAM1;    // Size in RAM1
  DWORD dwSize_RAM2;    // Size in RAM2
  DWORD dwSize_RAM3;    // Size in RAM3
  BYTE  bBoxLabel[CBIOS_LABEL_LEN];      // Box label
  DWORD dwDeveloperID;  // Developer ID
  BYTE  bMPI;           // Box is MPI formatted
  WORD  wType;          // Box type. See CBIOS_BOX_* defenitions
  DWORD dwSize_RAM4;    // Size in RAM4
  DWORD dwSize_RAM5;    // Size in RAM5
} PACKED CBIOS_BOX_INFO_ADV;

typedef struct
{
  WORD  wAppID;                // APPID
  WORD  wRAM1Size;             // Size in RAM1
  WORD  wRAM2Size;             // Size in RAM2
  WORD  wRAM3Size;             // Size in RAM3
} PACKED CBIOS_APP_INFO;

#define CBIOS_BOX_ATTACHED 1
#define CBIOS_BOX_REMOVED  2
#define CBIOS_BOXES_CHANGED 3 //for network

typedef struct
{
  void* pNotificationParam;
  DWORD dwNotificationID;
  DWORD dwNotificationType;
} PACKED CBIOS_NOTIFY_DATA;

//RSA KEY definition
#define CBIOS_MAX_RSA_MODULUS_BITS 4096
#define CBIOS_MAX_RSA_MODULUS_LEN ((CBIOS_MAX_RSA_MODULUS_BITS + 7) / 8)
#define CBIOS_MAX_RSA_PRIME_BITS ((CBIOS_MAX_RSA_MODULUS_BITS + 1) / 2)
#define CBIOS_MAX_RSA_PRIME_LEN ((CBIOS_MAX_RSA_PRIME_BITS + 7) / 8)

#define CBIOS_RSA_MODULUS_LEN(bits) ((bits + 7) / 8)
#define CBIOS_RSA_PRIME_BITS(bits) ((bits + 1) / 2)
#define CBIOS_RSA_PRIME_LEN(bits) ((CBIOS_RSA_PRIME_BITS(bits) + 7) / 8)

#define CBIOS_RSA_PUBLIC_LEN(bits) (sizeof(WORD)+2*CBIOS_RSA_MODULUS_LEN(bits))
#define CBIOS_RSA_PRIVATE_LEN(bits) (sizeof(WORD)+3*CBIOS_RSA_MODULUS_LEN(bits)+3*CBIOS_RSA_PRIME_LEN(bits))


// Length of digit in bits
#define CBIOS_NN_DIGIT_BITS 32
// Length of digit in bytes
#define CBIOS_NN_DIGIT_LEN (CBIOS_NN_DIGIT_BITS / 8)
// Length of random pool
#define CBIOS_RAND_POOL_SIZE(x) (CBIOS_NN_DIGIT_LEN*((((x) + CBIOS_NN_DIGIT_BITS - 1) / CBIOS_NN_DIGIT_BITS) + 1))

//CBU2:
// maximum RSA key size
#define CBIOS_CBU2_MAX_RSA_KEY_LEN	256 // in bytes (2048 bits)
// memory usage of one RSA key in CBU2 memory (RAM4)
#define CBIOS_CBU2_RSA_KEY_SIZE		((CBIOS_CBU2_MAX_RSA_KEY_LEN * 2) + 14)

#define CBIOS_CBU2_RAM4_SIZE(x) ((x) * CBIOS_CBU2_RSA_KEY_SIZE)
#define CBIOS_CBU2_RAM4_KEYPAIRS(x) ((x) / CBIOS_CBU2_RSA_KEY_SIZE)

// AES key size
#define CBIOS_AES_KEY_LEN	16 // in bytes (128 bits)
#define CBIOS_AES_IV_LEN	16 // in bytes (128 bits)
// memory usage of one AES key in CBU2 memory (RAM5)
#define CBIOS_CBU2_AES_KEY_SIZE		(CBIOS_AES_KEY_LEN + 12)

#define CBIOS_CBU2_RAM5_SIZE(x) ((x) * CBIOS_CBU2_AES_KEY_SIZE)
#define CBIOS_CBU2_RAM5_KEYS(x) ((x) / CBIOS_CBU2_AES_KEY_SIZE)


/* RSA public and private key. */
typedef struct 
{
  unsigned int bits;                           /* length in bits of modulus */
  unsigned char modulus[CBIOS_MAX_RSA_MODULUS_LEN];                    /* modulus */
  unsigned char exponent[CBIOS_MAX_RSA_MODULUS_LEN];           /* public exponent */
} PACKED CBIOS_RSA_PUBLIC_KEY;

typedef struct 
{
  unsigned int bits;                           /* length in bits of modulus */
  unsigned char modulus[CBIOS_MAX_RSA_MODULUS_LEN];                    /* modulus */
  unsigned char publicExponent[CBIOS_MAX_RSA_MODULUS_LEN];     /* public exponent */
  unsigned char exponent[CBIOS_MAX_RSA_MODULUS_LEN];          /* private exponent */
  unsigned char prime[2][CBIOS_MAX_RSA_PRIME_LEN];               /* prime factors */
  unsigned char primeExponent[2][CBIOS_MAX_RSA_PRIME_LEN];   /* exponents for CRT */
  unsigned char coefficient[CBIOS_MAX_RSA_PRIME_LEN];          /* CRT coefficient */
} PACKED CBIOS_RSA_PRIVATE_KEY;

//CBU2:
typedef struct
{
  DWORD dwStructSize;
  WORD  wModulusLen; // in bytes, must be a multiple of four bytes
  WORD  wPrivExpLen; // in bytes, must be a multiple of four bytes
  BYTE  ubModulus[CBIOS_CBU2_MAX_RSA_KEY_LEN]; // most significant byte first
  BYTE  ubPrivExp[CBIOS_CBU2_MAX_RSA_KEY_LEN]; // most significant byte first
} PACKED CBIOS_RSA_KEY;

//CBU2 RSA:

//mode of keys:
#define CBIOS_RSA_PUBL_KEY  1
#define CBIOS_RSA_PRIV_KEY  2
//mode of padding
#define CBIOS_RSA_MARX_PADDING		0x00
#define CBIOS_RSA_RSAREF_PADDING	0x10

typedef struct
{
  DWORD dwStructSize;
  DWORD dwAccess; // low nibble: SetKey[Info]RSA, high nibble: CBIOS_(De/En)cryptRSAEx/GetKeyInfoRSA
  WORD  wBits;
} PACKED CBIOS_RSA_KEY_INFO;

typedef struct 
{
  DWORD dwStructSize;
  BYTE  ubKey[CBIOS_AES_KEY_LEN]; // most siginificant byte first
} PACKED CBIOS_AES_KEY;

//CBU2 AES
//mode of keys:
#define CBIOS_AES_OFB			0x01
#define CBIOS_AES_CBC_ENCRYPT	0x02
#define CBIOS_AES_CBC_DECRYPT	0x03

typedef struct
{
  DWORD dwStructSize;
  DWORD dwAccess; // low nibble: SetKey[Info]AES, high nibble: CBIOS_(De/En)cryptAES/GetKeyInfoAES
  WORD  wLen;
} PACKED CBIOS_AES_KEY_INFO;



typedef struct
{ 
  CHAR  szServerIp[48]; // server IP address like "127.0.0.1" with terminating zero
  WORD  wServerPort;    // server port for connection
  INT32 iBoxesNumber;   // number of attached boxes on server
} PACKED CBIOS_SERVER_INFO;

#define CBIOS_NET_LICENCE_RULE_DEFAULT      0 // licenses are locked without taking client’s IP/Terminal Session into account (default behavior – as implemented now)
#define CBIOS_NET_LICENCE_RULE_IP_1L        1 // "one seat per one IP", only one license will be locked for all calls coming from one IP address without notice on TS
							//(unlimited number of instances can be launched concurrently from one IP)
#define CBIOS_NET_LICENCE_RULE_MACADD_TS_1L 2 // "one seat per one <MAC+User+Add>" for dynamic IP, only one license will be locked for all calls coming from:
							//- one MAC address (unlimited number of instances can be launched concurrently from one MAC)
							//- one Terminal Session (unlimited number of instances can be launched concurrently in the same TS)
							//and additional checking (soft&hard components)
#define CBIOS_NET_LICENCE_RULE_IPADD_TS_1L  3 // "one seat per one <IP+MAC+User+Add>" for static IP, the same as CBIOS_NET_LICENCE_RULE_MACADD_TS_1L + IP
#define CBIOS_NET_LICENCE_RULE_MAX CBIOS_NET_LICENCE_RULE_IPADD_TS_1L
#define CBIOS_NET_LICENCE_IS_RULE_DEFAULT(rule) ((rule)==CBIOS_NET_LICENCE_RULE_DEFAULT)
#define CBIOS_NET_LICENCE_IS_VALID_RULE(rule) ( (BYTE)(rule)<=CBIOS_NET_LICENCE_RULE_MAX )

typedef struct
{
  DWORD dwStructSize;
  BYTE  bCallbacks; // Callbacks qty
} PACKED CBIOS_STATUS_INFO;

#pragma pack()

#ifdef __cplusplus
extern "C"
{
#endif

extern BYTE CBIOS_DEMO_DISTR_RSA_KEY_MODULUS[CBIOS_INTERNAL_RSA_LEN];
extern BYTE CBIOS_DEMO_DISTR_RSA_KEY_EXPONENT[CBIOS_INTERNAL_RSA_LEN];
extern BYTE CBIOS_DEMO_CLIENT_RSA_KEY_MODULUS[CBIOS_INTERNAL_RSA_LEN];
extern BYTE CBIOS_DEMO_CLIENT_RSA_KEY_EXPONENT[CBIOS_INTERNAL_RSA_LEN];

DWORD WINAPI CBIOS_SetLicensePath(const char* path, const DWORD len);
DWORD WINAPI CBIOS_Startup(void);
DWORD WINAPI CBIOS_Finish(void);

DWORD WINAPI CBIOS_StartupDLL(void); //obsolete, see CBIOS_StartupDLLEx
#ifdef WIN32
DWORD WINAPI CBIOS_StartupDLLEx(HMODULE hDll);
#endif
DWORD WINAPI CBIOS_FinishDLL(void);

DWORD WINAPI CBIOS_Connect(CHAR *ServerName, WORD wPort);
INT32 WINAPI CBIOS_ScanBoxes(void);
DWORD WINAPI CBIOS_Disconnect(void);

DWORD WINAPI CBIOS_GetBoxInfoI(INT32 iBoxIndex, CBIOS_BOX_INFO* pBoxInfo);
DWORD WINAPI CBIOS_GetBoxInfoAdvI(INT32 iBoxIndex, CBIOS_BOX_INFO_ADV* pBoxInfo);
DWORD WINAPI CBIOS_GetDeveloperIDI(INT32 iBoxIndex, DWORD* pdwDeveloperID);
DWORD WINAPI CBIOS_GetAppInfoI(INT32 iBoxIndex, INT32 iAppIndex, CBIOS_APP_INFO* pAppInfo);

DWORD WINAPI CBIOS_OpenByIndex(INT32 iBoxIndex, WORD wAppID);
DWORD WINAPI CBIOS_OpenByName(DWORD dwBoxName, WORD wAppID);
DWORD WINAPI CBIOS_OpenByLabel(const BYTE bBoxLabel[CBIOS_LABEL_LEN], WORD wAppID);
DWORD WINAPI CBIOS_OpenByApp(WORD wAppID);
DWORD WINAPI CBIOS_Close(void);

BOOL  WINAPI CBIOS_IsBoxLockedByOthersI(INT32 iBoxIndex);
BOOL  WINAPI CBIOS_IsBoxLockedByOthers(void);
DWORD WINAPI CBIOS_LockBox(void);
DWORD WINAPI CBIOS_LockBoxEx(DWORD dwTimeoutMsec);
DWORD WINAPI CBIOS_UnlockBox(void);

DWORD WINAPI CBIOS_LockLicence(void);//Ignores rules of extended network licensing
DWORD WINAPI CBIOS_LockLicenceExt(); //Use this function only if network extended licensing is needed
DWORD WINAPI CBIOS_ReleaseLicence(void); //both(ext or not) for session

DWORD WINAPI CBIOS_GetDriverLastError(void);
DWORD WINAPI CBIOS_CheckBox(void);
//---------------------------------------------------------

DWORD WINAPI CBIOS_GetAppInfo(INT32 iAppIndex, CBIOS_APP_INFO* pAppInfo);
DWORD WINAPI CBIOS_GetBoxInfo(CBIOS_BOX_INFO* pBoxInfo);
DWORD WINAPI CBIOS_GetBoxInfoAdv(CBIOS_BOX_INFO_ADV* pBoxInfo);
DWORD WINAPI CBIOS_GetDeveloperID(DWORD* pdwDeveloperID);

DWORD WINAPI CBIOS_SetLabel(const BYTE bBoxLabel[CBIOS_LABEL_LEN]);

DWORD WINAPI CBIOS_UPWLogin(BYTE bUPW[0x10]);
DWORD WINAPI CBIOS_APWLogin(BYTE bAPW[0x10]);
DWORD WINAPI CBIOS_Logout(void);

DWORD WINAPI CBIOS_GetHWRand(DWORD dwLen, PVOID pBuffer);
DWORD WINAPI CBIOS_ReadRAM1(DWORD dwAddress, DWORD dwLen, PVOID pBuffer, BYTE bPass[0x10]);
DWORD WINAPI CBIOS_WriteRAM1(DWORD dwAddress, DWORD dwLen, PVOID pBuffer, BYTE bPass[0x10]);
DWORD WINAPI CBIOS_ReadRAM2(DWORD dwAddress, DWORD dwLen, PVOID pBuffer, BYTE bPass[0x10]);
DWORD WINAPI CBIOS_WriteRAM2(DWORD dwAddress, DWORD dwLen, PVOID pBuffer, BYTE bPass[0x10]);
DWORD WINAPI CBIOS_ReadRAM3(DWORD dwAddress, DWORD dwLen, PVOID pBuffer, BYTE bPass[0x10]);
DWORD WINAPI CBIOS_WriteRAM3(DWORD dwAddress, DWORD dwLen, PVOID pBuffer, BYTE bPass[0x10]);
DWORD WINAPI CBIOS_SetUPW(BYTE bOldPass[0x10], BYTE bNewPass[0x10]);
DWORD WINAPI CBIOS_SetAPW(BYTE bOldPass[0x10], BYTE bNewPass[0x10]);
//---------------------------------------------------------
DWORD WINAPI CBIOS_SetKeySession(BYTE bKEY[0x10]);
DWORD WINAPI CBIOS_SetIVSession(BYTE bIV[0x10]);
DWORD WINAPI CBIOS_SetKeyPrivate(BYTE bAPW[0x10], BYTE bKEY[0x10]);
DWORD WINAPI CBIOS_SetIVPrivate(BYTE bAPW[0x10], BYTE bIV[0x10]);
//---------------------------------------------------------
DWORD WINAPI CBIOS_CryptFixed(DWORD dwLen, PVOID pBuffer);
DWORD WINAPI CBIOS_CryptPrivate(DWORD dwLen, PVOID pBuffer);
DWORD WINAPI CBIOS_CryptSession(DWORD dwLen, PVOID pBuffer);
//---------------------------------------------------------
DWORD WINAPI CBIOS_EncryptRSA(DWORD dwKeyMemory, DWORD dwKeyOffset, PVOID pInBuffer,
                              DWORD dwInBufferLen, PVOID pOutBuffer, DWORD *dwOutBufferLen, BYTE bPass[16]);
DWORD WINAPI CBIOS_DecryptRSA(DWORD dwKeyMemory,DWORD dwKeyOffset, PVOID pInBuffer,
                               DWORD dwInBufferLen, PVOID pOutBuffer, DWORD *dwOutBufferLen, BYTE bPass[16]);

//---------------------------------------------------------
DWORD WINAPI CBIOS_GetKeyPublicRSA(DWORD dwKeyMemory,DWORD dwKeyOffset,CBIOS_RSA_PUBLIC_KEY *pRSAKey, BYTE bPass[16]);
DWORD WINAPI CBIOS_GetKeyPrivateRSA(DWORD dwKeyMemory,DWORD dwKeyOffset,CBIOS_RSA_PRIVATE_KEY *pRSAKey, BYTE bPass[16]);
DWORD WINAPI CBIOS_SetKeyPublicRSA(DWORD dwKeyMemory,DWORD dwKeyOffset,CBIOS_RSA_PUBLIC_KEY *pRSAKey, BYTE bPass[16]);
DWORD WINAPI CBIOS_SetKeyPrivateRSA(DWORD dwKeyMemory,DWORD dwKeyOffset,CBIOS_RSA_PRIVATE_KEY *pRSAKey, BYTE bPass[16]);
//---------------------------------------------------------
DWORD WINAPI CBIOS_GenerateKeyPairRSA(DWORD dwPublicKeyMemory, DWORD dwPublicKeyOffset,
                                       DWORD dwPrivateKeyMemory, DWORD dwPrivateKeyOffset, WORD bits, BYTE *randomPool, BYTE bPass[16]);
//CBIOS_..RSAEx - software implementation :
DWORD WINAPI CBIOS_GenerateKeyPairRSAEx(CBIOS_RSA_KEY * pRSAKeyPair, CBIOS_RSA_KEY * pRSAPublicKey, WORD bits, BYTE *randomPool);// pRSAPublicKey = NULL - optional parameter
//---------------------------------------------------------
DWORD WINAPI CBIOS_EncryptRSAEx(CBIOS_RSA_KEY * pRSAKey, DWORD dwMode, PVOID pInBuffer, DWORD dwInBufferLen, PVOID pOutBuffer, DWORD *pdwOutBufferLen);
DWORD WINAPI CBIOS_DecryptRSAEx(CBIOS_RSA_KEY * pRSAKey, DWORD dwMode, PVOID pInBuffer, DWORD dwInBufferLen, PVOID pOutBuffer, DWORD *pdwOutBufferLen);
//---------------------------------------------------------
DWORD WINAPI CBIOS_CBU2_SetKeyRSA(DWORD dwKeyIndex, CBIOS_RSA_KEY * pRSAKeyPair, CBIOS_RSA_KEY_INFO * pRSAKeyInfo);
DWORD WINAPI CBIOS_CBU2_SetKeyInfoRSA(DWORD dwKeyIndex, CBIOS_RSA_KEY_INFO * pRSAKeyInfo);
DWORD WINAPI CBIOS_CBU2_GetKeyInfoRSA(DWORD dwKeyIndex, CBIOS_RSA_KEY_INFO * pRSAKeyInfo);
DWORD WINAPI CBIOS_CBU2_LockKeyRSA(DWORD dwKeyIndex);
//---------------------------------------------------------
DWORD WINAPI CBIOS_EncryptInternalRSA(DWORD dwKeyID, PVOID pInBuffer, DWORD dwInBufferLen, PVOID pOutBuffer, DWORD *pdwOutBufferLen, BYTE bPass[16]);
DWORD WINAPI CBIOS_DecryptInternalRSA(DWORD dwKeyID, PVOID pInBuffer, DWORD dwInBufferLen, PVOID pOutBuffer, DWORD *pdwOutBufferLen, BYTE bPass[16]);
//For CBU2 (see SmarxOS-Manual):
DWORD WINAPI CBIOS_CBU2_EncryptRSA(DWORD dwKeyIndex, DWORD dwMode, PVOID pInBuffer, DWORD dwInBufferLen, PVOID pOutBuffer, DWORD *pdwOutBufferLen);
DWORD WINAPI CBIOS_CBU2_DecryptRSA(DWORD dwKeyIndex, DWORD dwMode, PVOID pInBuffer, DWORD dwInBufferLen, PVOID pOutBuffer, DWORD *pdwOutBufferLen);

void  WINAPI CBIOS_PrepareRSAKey(WORD bits, BYTE* pbModulus, BYTE* pbExponent, BYTE* pbRSAkey);
//=====================================================================================================================
DWORD WINAPI CBIOS_CBU2_SetKeyAES (DWORD dwKeyIndex, CBIOS_AES_KEY* pAESKey, CBIOS_AES_KEY_INFO * pAESKeyInfo);
DWORD WINAPI CBIOS_CBU2_SetKeyInfoAES (DWORD dwKeyIndex, CBIOS_AES_KEY_INFO * pAESKeyInfo);
DWORD WINAPI CBIOS_CBU2_GetKeyInfoAES (DWORD dwKeyIndex, CBIOS_AES_KEY_INFO * pAESKeyInfo);
DWORD WINAPI CBIOS_CBU2_LockKeyAES (DWORD dwKeyIndex);
DWORD WINAPI CBIOS_CBU2_CryptAES (DWORD dwKeyIndex, DWORD dwMode, PVOID pIV, PVOID pInBuffer, PVOID pOutBuffer, DWORD dwBufferLen);


//---------------------------------------------------------
DWORD WINAPI CBIOS_MD5Hash(PVOID pInBuffer, DWORD dwInBufferLen, PVOID pOutBuffer, DWORD *pdwOutBufferLen);
//=====================================================================================================================
DWORD WINAPI CBIOS_GetAppLicences(WORD wAppID, DWORD* pdwRuleId, DWORD* pdwNetLic);
DWORD WINAPI CBIOS_SetAppLicences(WORD wAppID, DWORD dwReserved, DWORD dwNetLic, BYTE bPass[0x10]);//dwReserved would be 0
DWORD WINAPI CBIOS_SetRuleAppLicences(WORD wAppID, BYTE bRuleId, BYTE bNetLic, BYTE bPass[0x10]);
DWORD WINAPI CBIOS_CheckAppLicence(WORD wAppID, DWORD* pdwRuleId, DWORD* pdwNetLic);
//=====================================================================================================================
DWORD WINAPI CBIOS_GetTransactionMark(DWORD* pdwTransactionMark);
DWORD WINAPI CBIOS_SetTransactionMark(DWORD dwTransactionMark);

DWORD WINAPI CBIOS_GetAppPolicyI(INT32 iBoxIndex, OUT BYTE AppPolicy[100]);

//=====================================================================================================================
typedef DWORD CALLBACK F_CBIOS_NOTIFY_CALLBACK(CBIOS_NOTIFY_DATA NotificationData);

DWORD WINAPI CBIOS_RegisterNotificationCallback(F_CBIOS_NOTIFY_CALLBACK* fNotify, void* pParam);
DWORD WINAPI CBIOS_UnregisterNotificationCallback(F_CBIOS_NOTIFY_CALLBACK* fNotify);
#ifdef WIN32
DWORD WINAPI CBIOS_RegisterNotificationMessage(HWND hWnd, UINT   message, LPARAM lParam);
DWORD WINAPI CBIOS_UnregisterNotificationMessage(HWND hWnd);
#endif // WIN32
// =====================================================================================================================
DWORD WINAPI CBIOS_GetSerialNumI(INT32 iBoxIndex, BYTE SerNum[16]);
DWORD WINAPI CBIOS_GetSerialNum(OUT BYTE bSerNum[16]);

DWORD WINAPI CBIOS_CBM_GetVolumePathI(INT32 iBoxIndex, OUT CHAR szVolumePath[128]);
DWORD WINAPI CBIOS_CBM_GetVolumePath(OUT CHAR szVolumePath[128]);

//==========================================================================
DWORD WINAPI CBIOS_ErrorToText(DWORD dwError, char* szError);

//==========================================================================
INT32 WINAPI CBIOS_ScanNetwork(DWORD dwScanTimeMs);
// Set/get UDP port used for network scanning (default port is used if not set)
DWORD WINAPI CBIOS_SetScanPort(WORD wUDPPort);
DWORD WINAPI CBIOS_GetScanPort(WORD* pwUDPPort);
// iServerIndex starts from 1
DWORD WINAPI CBIOS_GetServerInfo(INT32 iServerIndex, CBIOS_SERVER_INFO* pServerInfo);

typedef VOID CALLBACK F_CBIOS_SERVER_CALLBACK(PVOID pData, CBIOS_SERVER_INFO* pServerInfo, PVOID pReserved);
typedef VOID CALLBACK F_CBIOS_STATUS_CALLBACK(PVOID pData, DWORD dwStatus, PVOID pStatusData); //CBIOS_STATUS_INFO
DWORD WINAPI CBIOS_ScanNetworkEx(DWORD dwScanTimeMs, PVOID pData, F_CBIOS_SERVER_CALLBACK* fServer, F_CBIOS_STATUS_CALLBACK* fStatus);

// Allow to abort the operation (in "long delay" case)
typedef BOOL CALLBACK F_CBIOS_CONNECT_CALLBACK(PVOID pData, PVOID pReserved);
DWORD WINAPI CBIOS_ConnectEx(CHAR *ServerName, WORD wPort, F_CBIOS_CONNECT_CALLBACK* fConnect, PVOID pData);
//==========================================================================
DWORD WINAPI CBIOS_ClientLogin(CHAR * passw);

//==========================================================================

//==========================================================================
// RUMS API

DWORD WINAPI RUMS_GetRequest(CHAR *szProjectGuid, BOOL flRegenerate, CHAR* szDescription, BYTE* pData, DWORD* pdwDataSize);
DWORD WINAPI RUMS_IsGenerated(CHAR *szProjectGuid, BOOL* pbRes);
DWORD WINAPI RUMS_ExecuteActivationCode(CHAR *szProjectGuid, BYTE* pData, DWORD dwDataSize);

DWORD WINAPI RUMS_GetRequestLocal(DWORD dwBoxName, CHAR *szProjectGuid, CHAR* szDescription, BYTE* pData, DWORD* pdwDataSize);
DWORD WINAPI RUMS_ExecuteActivationCodeLocal(DWORD dwBoxName, CHAR *szProjectGuid, BYTE* pData, DWORD dwDataSize);

//==========================================================================

#ifdef __cplusplus
}
#endif

#endif //_CBIOS_H_
