/*****************************************************************************
  Copyright:   : MARX CryptoTech LP
  Project      : TEOS Data Objects API
  Created      : 05Jan2004
  Tools        : MS VS 6 - 2012
  Description  : TEOS DO API definitions
******************************************************************************
  VER         DATE        WHO   DETAIL                        
******************************************************************************
  1.00        05Jan2004   VS    Initial version
  2.00		  16Nov2010	  KU	AES & RSA DO (CBU2 support)
  2.10        07Feb2012   VG    DO_BIND added
  2.20        18Sep2012   VG    Basic server DO engine support, DO_BIND for server added
  2.30        22Jan2013   VG    DO_GEOLICENSE added
  2.50        25Feb2014   VG    CDOs added, DO_BIND_ACTIVATION_REQUEST_DATA and DO_KEY_REF changed
  2.60        24Feb2016   VG    CDO_MEMORY added
*****************************************************************************/

#ifndef _TEOSDO_H_
#define _TEOSDO_H_

// data object types
// Highly recommended to use TEOSDO_CDO_*
#define TEOSDO_EXPIRATION_DATE 1 // Expiration Date		
#define TEOSDO_EXPIRATION_DATE_AND_TIME \
							   2 // Expiration Date & Time
#define TEOSDO_NUMBER_OF_DAYS  3 // Number of Days		
#define TEOSDO_TIME_ALLOWED    4 // Time Allowed		
#define TEOSDO_COUNTER         5 // Usage Counter		
#define TEOSDO_PSW_HASH        6 // Password Hash		
#define TEOSDO_NET_LICENCE     8 // Network License		
#define TEOSDO_DOUBLE_WORD     9 // Double Word		
#define TEOSDO_LOC_LICENCE    10 // Local licence
#define TEOSDO_APP_NAME_HASH  11 // Application flie name hash		 
#define TEOSDO_APP_CS         12 // Application exe-file checksum
#define TEOSDO_MEMORY         13 // memory block
//AES & RSA DO support
#define TEOSDO_RSA		      14 // RSA key (CBU2 only)
#define TEOSDO_RSA_DISTRIBUTOR 15 // RSA Distributor (public) key
#define TEOSDO_RSA_CLIENT	  16 // RSA Client key
#define TEOSDO_AES		      17 // AES key (CBU2 only)
#define TEOSDO_AES_FIXED	  18 // AES Fixed key
#define TEOSDO_AES_PRIVATE	  19 // AES Private key
#define TEOSDO_AES_SESSION	  20 // AES Session key
//extended encryption support
#define TEOSDO_RSA_EX		  21 // RSA + Descriptor
#define TEOSDO_AES_EX		  22 // AES + Descriptor
#define TEOSDO_SIGNATURE	  23 // Signature

#define TEOSDO_NET_LICENCE_EX 24 // Network Extended License
//#define TEOSDO_NET_FLICENCE   25 // Network Floating License
#define TEOSDO_BINDING        26 // Binding to Device
#define TEOSDO_GEOLICENSE     27 // GeoLicense

#define TEOSDO_CDO_EXPIRATION_DATE_AND_TIME	(100+TEOSDO_EXPIRATION_DATE_AND_TIME)
#define TEOSDO_CDO_NUMBER_OF_DAYS			(100+TEOSDO_NUMBER_OF_DAYS)
#define TEOSDO_CDO_TIME_ALLOWED				(100+TEOSDO_TIME_ALLOWED)
#define TEOSDO_CDO_COUNTER					(100+TEOSDO_COUNTER)
#define TEOSDO_CDO_PSW_HASH					(100+TEOSDO_PSW_HASH)
#define TEOSDO_CDO_APP_NAME_HASH			(100+TEOSDO_APP_NAME_HASH)
#define TEOSDO_CDO_APP_CS					(100+TEOSDO_APP_CS)
#define TEOSDO_CDO_BINDING					(100+TEOSDO_BINDING)
#define TEOSDO_CDO_GEOLICENSE				(100+TEOSDO_GEOLICENSE)
#define TEOSDO_CDO_MEMORY                   (100+TEOSDO_MEMORY)


typedef struct DO_NET_LICENCE_EX_DATA
{
	BYTE			 bNetLic; 
	BYTE			 bRuleId; 
	WORD			 wReserved;
} DO_NET_LICENCE_EX_DATA;

//typedef struct DO_NET_FLICENCE_HOST_DATA
//{
//	BYTE			 bMaxDays; // max time interval (duration)
//	BYTE			 bMaxLic;  // max number of seats for check out
//} DO_NET_FLICENCE_HOST_DATA;

#define TEOSDO_STATUS_UNBOUND 0
#define TEOSDO_STATUS_BOUND 1
#define TEOSDO_STATUS_ACTIVATED 2
typedef struct DO_BIND_DATA_OUT
{
	BYTE bStatus;
	BYTE bReserved[3];
} DO_BIND_DATA_OUT;
typedef BYTE DO_BIND_ACTIVATION_REQUEST_DATA[20];


//TEOSDO_RSA_EX data for TEOSDO_SET method
#define DO_RSA_EX_KEY_INDEX_NONE -1
#define DO_RSA_EX_KEY_INDEX_DISTR -2
#define DO_RSA_EX_KEY_INDEX_CIENT -3

typedef struct DO_RSA_EX_DATA
{
	DWORD			 dwKeyIndex; //RSA key index, "absent" = -1, Distributor public = -2, Client private = -3
	DWORD			 dwKeyMode; //CBIOS_RSA_PUBL_KEY 1, CBIOS_RSA_PRIV_KEY  2 
							   //CBIOS_RSA_MARX_PADDING 0x00, CBIOS_RSA_RSAREF_PADDING 0x10
	DWORD			 dwUpdateDescr;// version/update rule
	DWORD			 dwBoxName;
} DO_RSA_EX_DATA;

//TEOSDO_AES_EX data for TEOSDO_SET method
#define DO_AES_EX_KEY_INDEX_NONE -1
#define DO_AES_EX_KEY_INDEX_FIXED -2
#define DO_AES_EX_KEY_INDEX_PRIVT -3
#define DO_AES_EX_KEY_INDEX_SESSN -4

typedef struct DO_AES_EX_DATA
{
	DWORD			 dwKeyIndex; //AES key index, "absent" = -1, Fixed = -2, Private = -3, Session = -4
	DWORD			 dwKeyMode; //CBIOS_AES_OFB 0x01, CBIOS_AES_CBC_ENCRYPT 0x02, CBIOS_AES_CBC_DECRYPT 0x03
							   //Calc IV method: 0x00 - default (provided), 0x10 - method #2, ...
	DWORD			 dwUpdateDescr;// version/update rule
	DWORD			 dwReserved;
} DO_AES_EX_DATA;

//TEOSDO_SIGNATURE data for TEOSDO_SET method
typedef struct DO_SIGNATURE_DATA
{
  DO_RSA_EX_DATA keyA;
  DO_RSA_EX_DATA keyB;
  DWORD			 dwHashType;		// 0 - MD5Hash
  DWORD			 dwTimeStampSize;	// 0 - no timestamp
   									// 4 - __time32_t									
} DO_SIGNATURE_DATA;


#define TEOSDO_GEO_DEF_TIMEOUT 5000

//If parameter TEOSDO_GEO_DOUBLE_UNINIT for DO_GEO_DATA will be assigned to:
// errorRadius   - latitude, longitude will be ignored
// altitudeError - altitude will be ignored
#define TEOSDO_GEO_DOUBLE_UNINIT -(DOUBLE)MAXDWORD

#define TEOSDO_GEO_BIND_LATLONG_ID      1
#define TEOSDO_GEO_BIND_CIVADDR_ID      2
#define TEOSDO_GEO_BIND_LATLONG         4
#define TEOSDO_GEO_BIND_ALTITUDE        8
#define TEOSDO_GEO_BIND_ADDRLINE1  0x0010
#define TEOSDO_GEO_BIND_ADDRLINE2  0x0020
#define TEOSDO_GEO_BIND_CITY       0x0040
#define TEOSDO_GEO_BIND_COUNTRYREG 0x0080
#define TEOSDO_GEO_BIND_POSTCODE   0x0100
#define TEOSDO_GEO_BIND_STATEPROV  0x0200
#define TEOSDO_GEO_BIND_CIVADDR_ALL  (TEOSDO_GEO_BIND_ADDRLINE1 | TEOSDO_GEO_BIND_ADDRLINE2 |\
	TEOSDO_GEO_BIND_CITY | TEOSDO_GEO_BIND_COUNTRYREG | TEOSDO_GEO_BIND_POSTCODE | TEOSDO_GEO_BIND_STATEPROV)

typedef struct DO_GEO_DATA
{
	WORD wBindFlag; //TEOSDO_GEO_BIND_*
	BYTE desiredLatLongAccuracy, desiredCivicAddressAccuracy; // IN/OUT, possible desired accuracy types, see Location API on MSDN
	DWORD dwReserved;
	DOUBLE latitude, longitude,
		errorRadius,//a distance from the reported location, in meters. Combined with the location reported as the origin, this radius describes a circle in which the actual location is probably located
		altitude, //in meters;
		altitudeError; //the altitude error, in meters
	WCHAR latLongSensorID[128], civicAddressSensorID[128];
	WCHAR AddressLine1[128], AddressLine2[128]; //line of a street address
	WCHAR City[128];
	WCHAR CountryRegion[128];
	WCHAR PostalCode[128];
	WCHAR StateProvince[128];
	DWORD detailLevel; //Reserved - the field contains the most accurate information available
	DWORD dwReserved2;
} DO_GEO_DATA;

typedef struct DO_GEO_DATA_OUT
{
	BYTE bStatus;
	BYTE bNeedBind;
	WORD reserved;
	DWORD dwTimestamp;// timestamp of TEOSDO_SET or TEOSDO_BIND
} DO_GEO_DATA_OUT;

typedef struct DO_GEO_DATA_IN_OUT
{
	DO_GEO_DATA geoData; //IN/OUT
	DWORD dwTimeoutMsec; //IN
	LONG  extError; //detailed error OUT
} DO_GEO_DATA_IN_OUT;


#pragma pack(1)

typedef struct CDO_SIGNED_DATA
{
	DWORD dwDataSize, dwSignSize;
	BYTE signature[1024];
	BYTE reserved2[7];
	BYTE data[1];
} CDO_SIGNED_DATA;

#define CDO_SIGNED_DATA_SIZE(dataSize) (sizeof(CDO_SIGNED_DATA) + ((dataSize)?(dataSize)-1:0))

#pragma pack()

// operation codes
#define TEOSDO_SET              1
#define TEOSDO_INC              2
#define TEOSDO_DEC              3
#define TEOSDO_CLEAR            4
#define TEOSDO_UNLIMITED        5
#define TEOSDO_VERIFY           6
#define TEOSDO_GET				7 // RE: added 21-OCT-2006, needed for Web API
//AES & RSA DO support
#define TEOSDO_SET_KEY          8  // (RSA and AES)
#define TEOSDO_CLEAR_KEY        9  // (RSA and AES)
#define TEOSDO_ENCRYPT			10 // (RSA and AES)
#define TEOSDO_DECRYPT			11 // (RSA only) 
#define TEOSDO_GENERATE			12 // Generate and set RSA key-pair, return public key (RSA only)
#define TEOSDO_SIGN				13 // = Encrypt private (RSA only)
#define TEOSDO_VALIDATE_SIGNATURE 14 // = Decrypt public (RSA only)
#define TEOSDO_CALCULATE_DIGITAL_SIGNATURE 15 // = Signature DO only
#define TEOSDO_VALIDATE_DIGITAL_SIGNATURE 16 // = Signature DO only
#define TEOSDO_UNLIMITED_EX     17

#define TEOSDO_BIND         TEOSDO_INC

//#define TEOSDO_ACTIVATE_LEASE	TEOSDO_DEC // for host-box
//#define TEOSDO_DEACTIVATE_LEASE	TEOSDO_INC // for host-box

// field ID codes
#define TEOSDO_ID               1
#define TEOSDO_TYPE             2
#define TEOSDO_RAM              3
#define TEOSDO_OFFSET           4
#define TEOSDO_SIZE             5
#define TEOSDO_NAME             6
#define TEOSDO_DATA             7
#define TEOSDO_DATASIZE         8
#define TEOSDO_FLAGS			9

// parameter values

#define TEOSDO_LOCAL_IN_NET_MODE 1 //for TEOS_DoCreateReference

// TEOSDO_EXPIRATION_DATE and TEOSDO_EXPIRATION_DATE_AND_TIME support
#define TEOSDO_DATE_AS_STRING	0	// get/set expiration date as string, like "31 DEC 1999" - 11 symbols total (needs buffer of 12 bytes long to get)
									// or set expiration date with timestamp as string, like "31 DEC 1999, 13 DEC 1999" - 24+ symbols total
									// or set expiration date and time as string (TEOSDO_EXPIRATION_DATE_AND_TIME only), like "31 DEC 1999 23:59:59" - 20+ symbols total
									// or set expiration date and time with timestamp as string (TEOSDO_EXPIRATION_DATE_AND_TIME only), like "31 DEC 1999 23:59:59, 13 DEC 1999" - 33+ symbols total
#define TEOSDO_DATE_AS_SYSTEMTIME \
								1	// set expiration date as SYSTEMTIME struct (two consecutive SYSTEMTIME structs - for expiration date and timestamp)
#define TEOSDO_DATE_AS_DWORD	1	// get expiration date as dword (days left)
#define TEOSDO_DATE_AS_TIME32	2	// get (and also inc/dec) expiration date as __time32_t (seconds left)
#define TEOSDO_DATE_AS_STRING_WITH_TIME	\
								4	// get expiration date and time as string (like "31 DEC 1999 23:59:59" - 20 symbols total, needs 21 bytes buffer)
// others
#define TEOSDO_NUMBER_OF_DAYS_AS_EXPIRATION_DATE \
								2 // specifies that we actually deel with _TEOSDO_EXPIRATION_DATE_RELATIVE dataobj
#define TEOSDO_NUMBER_OF_DAYS_AS_EXPIRATION_DATE_GET_BEFORE_FISRT_RUN \
								3 // specifies that we actually deel with _TEOSDO_EXPIRATION_DATE_RELATIVE dataobj
								  // used to check the state of dataobject (returns "1" if it is before "first run" and "0" otherwise)
#define TEOSDO_BIND_GENERATE_ACTIVATION_REQUEST 1 //for DO_GET
#define TEOSDO_BIND_ACTIVATE     1 //for DO_BIND
#define TEOSDO_GEO_LOCATION      1 //for TEOSDO_GEOLICENSE

// error codes
#define TEOS_ERR_NOT_IMPLEMENTED         0x41
#define TEOS_ERR_WRONG_PARAMETER         0x42
#define TEOS_ERR_BUFFER_TOO_SHORT        0x43
#define TEOS_ERR_CORRUPTED_DATA          0x44
#define TEOS_ERR_MODIFIED_DATE           0x45
#define TEOS_ERR_EXPIRED                 0x46
#define TEOS_ERR_NO_HANDLES              0x47
#define TEOS_ERR_NO_MEMORY               0x48
#define TEOS_ERR_ALREADY_EXISTS          0x4B
#define TEOS_ERR_DO_NOT_FOUND            0x49
#define TEOS_ERR_NO_USAGE_COUNTER        0x4A
#define TEOS_ERR_CRC_ERROR               0x4C
#define TEOS_ERR_FILE_NOT_FOUND          0x4D
#define TEOS_ERR_FILE_ACCESS_ERROR       0x4E
#define TEOS_ERR_REF_NO_BASE_FIELD       0x4F // map: reference not created
#define TEOS_ERR_READ_ONLY               0x50 //e.g. result of SET for TEOSDO_AES_FIXED
#define TEOS_ERR_NOT_SUPPORTED	         0x51

#define TEOSDO_RESERVED                    0x100 // reserved

typedef BYTE PASSW[0x10];
#define TEOSDO_UNLIMITED_VALUE 0xFFFFFFFF

#define SECADAY 86400

#ifdef __cplusplus
extern "C"
{
#endif

DWORD WINAPI TEOS_ErrorToText(DWORD dwError, char* szError);

// TEOS DO MAP
DWORD WINAPI TEOS_DoLoadMap(
                        BYTE* pData, 
                        DWORD dwSize);
DWORD WINAPI TEOS_DoLoadMapFile(const char* szFileName);//Implemented by RD
DWORD WINAPI TEOS_DoSaveMap(
                        BYTE* pData, 
                        DWORD *pdwSize);
DWORD WINAPI TEOS_DoSaveMapFile(const char* szFileName);//Implemented by RD

//FLAGS parameter support, function without "Ex" - for backward compatibility
DWORD WINAPI TEOS_DoSaveMapEx(
                        BYTE* pData, 
                        DWORD *pdwSize);
DWORD WINAPI TEOS_DoSaveMapFileEx(const char* szFileName);//Implemented by RD

DWORD WINAPI TEOS_DoCreateReference(
                        DWORD dwDoID,
                        DWORD dwDoType, 
                        DWORD dwRAMNumber, 
                        DWORD dwOffset,
                        DWORD dwFlags);

DWORD WINAPI TEOS_DoDeleteReference(DWORD dwDoID);

DWORD WINAPI TEOS_DoClearReferences(void);

INT   WINAPI TEOS_DoRefCount(void);

DWORD WINAPI TEOS_DoRefFindByIndex(
                        INT iRefIndex /*1...n*/,
                        DWORD *pdwDoID);

DWORD WINAPI TEOS_DoRefFindByName(
                        char *szName,
                        DWORD *pdwDoID);


DWORD WINAPI TEOS_DoRefSetBaseParameters(
                        DWORD dwDoID, 
                        DWORD dwRAMNumber, 
                        DWORD dwOffset,
                        DWORD dwReserved);

DWORD WINAPI TEOS_DoRefGetBaseParameters(
                        DWORD dwDoID, 
                        DWORD *pdwDoType, 
                        DWORD *pdwRAMNumber, 
                        DWORD *pdwOffset,
                        DWORD *pdwFlags);

DWORD WINAPI TEOS_DoRefSetParameter(
                        DWORD dwDoID,
                        DWORD dwParameterID,
                        BYTE  *pData,
                        DWORD dwSize);

DWORD WINAPI TEOS_DoRefGetParameter(
                        DWORD dwDoID,
                        DWORD dwParameterID,
                        BYTE  *pData,
                        DWORD *pdwSize);

DWORD WINAPI TEOS_DoGet(DWORD dwDoID,
                        BYTE* pData,
                        DWORD* pdwDataSize,
                        DWORD dwParameter,
                        PASSW bPass);

DWORD WINAPI TEOS_DoSet(DWORD dwDoID,
                        BYTE* pData,
                        DWORD dwDataSize,
                        DWORD dwParameter,
                        PASSW bPass);

DWORD WINAPI TEOS_DoOperation(DWORD dwDoID,
                        DWORD dwOperation,
                        BYTE* pData,
                        DWORD dwDataSize,
                        DWORD dwParameter,
                        PASSW bPass);

DWORD WINAPI TEOS_DoInc(DWORD dwDoID,
                        BYTE* pData,
                        DWORD dwDataSize,
                        DWORD dwParameter,
                        PASSW bPass);

DWORD WINAPI TEOS_DoDec(DWORD dwDoID,
                        BYTE* pData,
                        DWORD dwDataSize,
                        DWORD dwParameter,
                        PASSW bPass);

DWORD WINAPI TEOS_DoClear(DWORD dwDoID,
                        PASSW bPass);

DWORD WINAPI TEOS_DoUnlimited(DWORD dwDoID,
                        PASSW bPass);

DWORD WINAPI TEOS_DoUnlimitedEx(DWORD dwDoID,
						BYTE* pData,
                        DWORD dwDataSize,
                        DWORD dwParameter,
                        PASSW bPass);

DWORD WINAPI TEOS_DoVerify(DWORD dwDoID,
                        BYTE* pData,
                        DWORD dwDataSize,
                        DWORD dwParameter,
                        PASSW bPass);

// RSA & AES support

//pKey is
//	CBIOS_RSA_KEY* - for TEOSDO_RSA
//	BYTE [0x20]	   - for TEOSDO_AES_PRIVATE and TEOSDO_AES_SESSION
//	CBIOS_AES_KEY* - for TEOSDO_AES
DWORD WINAPI TEOS_DoSetKey(DWORD dwDoID
					    , PVOID /* CBIOS_RSA_KEY/CBIOS_AES_KEY * /BYTE [0x20] */ pKey
						, PVOID /* CBIOS_RSA_KEY_INFO/CBIOS_AES_KEY_INFO * */ pKeyInfo
                        , PASSW bPass);

DWORD WINAPI TEOS_DoClearKey(DWORD dwDoID
						, PVOID /*CBIOS_RSA_KEY_INFO/CBIOS_AES_KEY_INFO * */ pKeyInfoNew
                        , PASSW bPass);

/// AES specific

DWORD WINAPI TEOS_DoCryptAES(DWORD dwDoID
						, DWORD dwMode
					    , BYTE* pIV
					    , BYTE* pInBuffer
					    , BYTE* pOutBuffer, DWORD dwBufferLen
                        , PASSW bPass);

///obsolete
DWORD WINAPI TEOS_DoCryptAESEx(DWORD dwDoID
					    , BYTE* pIV
					    , BYTE* pInBuffer
					    , BYTE* pOutBuffer, DWORD dwBufferLen
                        , PASSW bPass);

DWORD WINAPI TEOS_DoEncryptAES(DWORD dwDoID
					    , BYTE* pIV
					    , BYTE* pInBuffer
					    , BYTE* pOutBuffer, DWORD dwBufferLen
                        , PASSW bPass);

DWORD WINAPI TEOS_DoDecryptAES(DWORD dwDoID
					    , BYTE* pIV
					    , BYTE* pInBuffer
					    , BYTE* pOutBuffer, DWORD dwBufferLen
                        , PASSW bPass);

/// RSA specific

DWORD WINAPI TEOS_DoGenerate(DWORD dwDoID
					    , PVOID /*CBIOS_RSA_KEY/CBIOS_AES_KEY * */ OUT pRSAPublicKey 
						, PVOID /*CBIOS_RSA_KEY_INFO/CBIOS_AES_KEY_INFO * */ pRSAKeyInfo
                        , PASSW bPass);

DWORD WINAPI TEOS_DoGenerateEx(DWORD dwDoID
						, DWORD dwBitsQty
					    , PVOID /*CBIOS_RSA_KEY/CBIOS_AES_KEY * */ OUT pRSAPublicKey 
						, PVOID /*CBIOS_RSA_KEY_INFO/CBIOS_AES_KEY_INFO * */ pRSAKeyInfo
                        , PASSW bPass);

DWORD WINAPI TEOS_DoEncryptRSA(DWORD dwDoID
						, DWORD dwMode
					    , BYTE* pInBuffer, DWORD dwInBufferLen
						, BYTE* pOutBuffer, DWORD *pdwOutBufferLen
                        , PASSW bPass);

///obsolete
DWORD WINAPI TEOS_DoEncryptRSAEx(DWORD dwDoID
					    , BYTE* pInBuffer, DWORD dwInBufferLen
						, BYTE* pOutBuffer, DWORD *pdwOutBufferLen
                        , PASSW bPass);

DWORD WINAPI TEOS_DoEncryptPublic(DWORD dwDoID
					    , BYTE* pInBuffer, DWORD dwInBufferLen
						, BYTE* pOutBuffer, DWORD *pdwOutBufferLen
                        , PASSW bPass);

DWORD WINAPI TEOS_DoEncryptPrivate(DWORD dwDoID
					    , BYTE* pInBuffer, DWORD dwInBufferLen
						, BYTE* pOutBuffer, DWORD *pdwOutBufferLen
                        , PASSW bPass);

DWORD WINAPI TEOS_DoDecryptRSA(DWORD dwDoID
						, DWORD dwMode
					    , BYTE* pInBuffer, DWORD dwInBufferLen
						, BYTE* pOutBuffer, DWORD *pdwOutBufferLen
                        , PASSW bPass);

///obsolete
DWORD WINAPI TEOS_DoDecryptRSAEx(DWORD dwDoID
					    , BYTE* pInBuffer, DWORD dwInBufferLen
						, BYTE* pOutBuffer, DWORD *pdwOutBufferLen
                        , PASSW bPass);

DWORD WINAPI TEOS_DoDecryptPublic(DWORD dwDoID
					    , BYTE* pInBuffer, DWORD dwInBufferLen
						, BYTE* pOutBuffer, DWORD *pdwOutBufferLen
                        , PASSW bPass);

DWORD WINAPI TEOS_DoDecryptPrivate(DWORD dwDoID
					    , BYTE* pInBuffer, DWORD dwInBufferLen
						, BYTE* pOutBuffer, DWORD *pdwOutBufferLen
                        , PASSW bPass);

// some extra functions...

DWORD WINAPI TEOS_DoSign(DWORD dwDoID
						, DWORD dwMode
					    , BYTE* pInBuffer, DWORD dwInBufferLen
						, BYTE* pOutBuffer, DWORD *pdwOutBufferLen
                        , PASSW bPass);

DWORD WINAPI TEOS_DoValidateSignature(DWORD dwDoID
						, DWORD dwMode
					    , BYTE* pInBuffer, DWORD dwInBufferLen
						, BYTE* pOutBuffer, DWORD *pdwOutBufferLen
                        , PASSW bPass);

DWORD WINAPI TEOS_DoCalculateFileSignature( DWORD dwDoID
						, DWORD dwMode
						, PVOID IN pRSAKey
						, const char* szSourceFileName
						, const char* szSignatureFileName
						, PASSW bPass);

DWORD WINAPI TEOS_DoValidateFileSignature( DWORD dwDoID
						, DWORD dwMode
						, const char* szSourceFileName
						, const char* szSignatureFileName
						, PASSW bPass);

// Signature DO support
DWORD WINAPI TEOS_DoSetKeys(DWORD dwDoID
						, PVOID /*CBIOS_RSA_KEY **/ pRSAKeyAPair
						, PVOID /*CBIOS_RSA_KEY_INFO **/ pRSAKeyAInfo
						, PVOID /*CBIOS_RSA_KEY **/ pRSAKeyBPair
						, PVOID /*CBIOS_RSA_KEY_INFO **/ pRSAKeyBInfo
                        , PASSW bPass);

DWORD WINAPI TEOS_DoGenerateA(DWORD dwDoID
						, DWORD dwBitsQty
					    , PVOID /*CBIOS_RSA_KEY/CBIOS_AES_KEY * */ OUT pRSAPublicKey 
						, PVOID /*CBIOS_RSA_KEY_INFO/CBIOS_AES_KEY_INFO * */ pRSAKeyInfo
                        , PASSW bPass);

DWORD WINAPI TEOS_DoGenerateB(DWORD dwDoID
						, DWORD dwBitsQty
					    , PVOID /*CBIOS_RSA_KEY/CBIOS_AES_KEY * */ OUT pRSAPublicKey 
						, PVOID /*CBIOS_RSA_KEY_INFO/CBIOS_AES_KEY_INFO * */ pRSAKeyInfo
                        , PASSW bPass);

DWORD WINAPI TEOS_DoCalculateDigitalSignature( DWORD dwDoID
					    , IN BYTE* pbSourceBuf, IN DWORD dwSourceBufLen
					    , IN OUT BYTE* pbSigBuf, IN OUT DWORD * pdwSigBufLen 
					    , PASSW bPass );

DWORD WINAPI TEOS_DoValidateDigitalSignature( DWORD dwDoID
					   , IN BYTE* pbSourceBuf, IN DWORD dwSourceBufLen
					   , IN BYTE* pbSigBuf, IN DWORD dwSigBufLen 
					   , OUT DWORD * pdwTimeStamp
					   , PASSW bPass );

DWORD WINAPI TEOS_DoCalculateDigitalSignatureF( DWORD dwDoID
					   , IN const char* szSourcePath
					   , IN const char* szSigPath //= 0 
					   , PASSW bPass );

DWORD WINAPI TEOS_DoValidateDigitalSignatureF( DWORD dwDoID
					   , IN const char* szSourcePath
					   , IN const char* szSigPath //= 0 
					   , OUT DWORD * pdwTimeStamp
					   , PASSW bPass );

#define KEY_REF_KEY_TYPE_NONE 0
#define KEY_REF_KEY_TYPE_DESCR 1
#define KEY_REF_KEY_TYPE_EXTNL 2

typedef struct DO_KEY_REF
{
	BYTE keyType; // 0 - Absent
				   // 1 - Descriptor ( keyDescriptor as DO_RSA_EX_DATA* )
				   // 2 - External ( keyRSA as CBIOS_RSA_KEY* + Mode from keyDescriptor )
	BYTE reserved[7];
	PVOID keyDescriptor; //DO_RSA_EX_DATA*, mandatory field
    PVOID keyRSA; //CBIOS_RSA_KEY*, optional field (keyType select)
} DO_KEY_REF;

#define TEOSDO_SIGN_ALG_TYPE_BOXNAME 1

DWORD WINAPI TEOS_CalculateDigitalSignature( DO_KEY_REF *pKeyA, DO_KEY_REF *pKeyB
										   , DWORD dwHashAlgType //=0 | 1
										   , DWORD dwTimeStampSize //=4 | 0
										   , IN BYTE* pbSourceBuf, IN DWORD dwSourceBufLen
										   , IN OUT BYTE* pbSigBuf, IN OUT DWORD * pdwSigBufLen 
										   , PASSW bPass );

DWORD WINAPI TEOS_ValidateDigitalSignature( DO_KEY_REF *pKeyA, DO_KEY_REF *pKeyB
										  , IN BYTE* pbSourceBuf, IN DWORD dwSourceBufLen
										  , IN BYTE* pbSigBuf, IN DWORD dwSigBufLen 
										  , OUT DWORD * pdwTimeStamp
										  , PASSW bPass );

DWORD WINAPI TEOS_CalculateDigitalSignatureF( DO_KEY_REF *pKeyA, DO_KEY_REF *pKeyB
										   , DWORD dwHashAlgType //= 0
										   , DWORD dwTimeStampSize //=4 | 0
										   , IN const char* szSourcePath
										   , IN const char* szSigPath //= 0 
										   , PASSW bPass );

DWORD WINAPI TEOS_ValidateDigitalSignatureF( DO_KEY_REF *pKeyA, DO_KEY_REF *pKeyB
										  , IN const char* szSourcePath
										  , IN const char* szSigPath //= 0 
										  , OUT DWORD * pdwTimeStamp
										  , PASSW bPass );


VOID WINAPI TEOS_BoundStatusToText(BYTE bStatus, char* szStatus);

#ifdef __cplusplus
}
#endif

#endif
