#include <windows.h>
#include <stdio.h>
#include "smarx_utils.h"
#include "cbios.h"
#include "teosdo.h"
#include "CDO_helpers.h"
#include "instr.h"

#define TEOSDO_ENC_UPW_ID 1001
#define TEOSDO_ENC_UPW_OFFSET 0
#define TEOSDO_ENC_UPW_RAM 3
#define TEOSDO_SESSION_AES_ID 1002
#define TEOSDO_SESSION_AES_RAM 3
#define TEOSDO_CDO_SIGNATURE_SIZE 12
#define UPW_LENGTH 16

// 
INT32 gl_iBoxIndex = 1;
WORD gl_wTestAppId = 101;


BYTE bAPW[] = SZ_DEMO_APW;
BYTE bUPW[] = SZ_DEMO_UPW;

CBIOS_RSA_KEY rsaPublicKey, rsaPrivateKey;

// These are the keys that are used in a programming(2) and a test(3) mode
// These keys can be overwritten in a generate(1) mode
BYTE bPrivRsaKeyModulus[64] =
{
0xcc, 0xa0, 0x25, 0xda, 0xa6, 0x0c, 0xa5, 0x0b, 0xf0, 0x19, 0xd5, 0x5a, 0xbd, 0x13, 0xa5, 0x47,
0xc6, 0xb3, 0x90, 0x19, 0x64, 0xee, 0x3f, 0x00, 0x5a, 0xc8, 0x9e, 0xab, 0x03, 0xcb, 0x5a, 0x12,
0xa4, 0x08, 0x95, 0x17, 0xdb, 0x90, 0x35, 0xc2, 0xe5, 0x24, 0x04, 0x22, 0x01, 0x97, 0x69, 0x7c,
0x61, 0xd5, 0xb7, 0xba, 0x0b, 0x5d, 0x62, 0x9a, 0xac, 0x8a, 0x72, 0x57, 0x9a, 0x73, 0x5c, 0xa7
};
BYTE bPrivRsaKeyExponent[64] =
{
0xb4, 0x2d, 0x94, 0x07, 0x6a, 0x4d, 0x1a, 0x0b, 0x5f, 0xf8, 0xaf, 0x58, 0x3d, 0xc3, 0x0d, 0xb7,
0x1c, 0xec, 0xc1, 0xc6, 0xca, 0x8b, 0xca, 0x67, 0xda, 0x88, 0x6a, 0xc6, 0x35, 0x0a, 0x1b, 0x1d,
0x59, 0xc6, 0x3b, 0xb7, 0x45, 0xb0, 0xb3, 0x6c, 0xc8, 0x30, 0xe8, 0x9c, 0xc7, 0x9d, 0x77, 0xe5,
0xd1, 0xe1, 0x97, 0x54, 0x80, 0x95, 0x6a, 0xe8, 0x52, 0x35, 0x4c, 0xb6, 0x4d, 0xaa, 0x39, 0x41
};

BYTE bRsaKeyModulus[64] =
{
0x20, 0x59, 0xe1, 0x3a, 0xfd, 0x97, 0xd1, 0x9d, 0xd6, 0x9e, 0x22, 0x75, 0x17, 0x1f, 0x96, 0x1b,
0xae, 0x96, 0x6c, 0x81, 0x1e, 0x92, 0x6f, 0x42, 0x15, 0x05, 0x71, 0xe3, 0x3e, 0x82, 0x2a, 0x2a,
0x28, 0xf4, 0x82, 0x96, 0xf9, 0x31, 0x14, 0x6a, 0xb2, 0x94, 0x7c, 0x98, 0x5b, 0xc9, 0xff, 0xd4,
0x45, 0xeb, 0x5e, 0x3d, 0xfb, 0xef, 0x37, 0x25, 0x9b, 0x9b, 0x84, 0x61, 0xe9, 0x8e, 0xce, 0xa3
};
BYTE bRsaKeyExponent[64] =
{
0xec, 0xf9, 0xc4, 0xe0, 0x5b, 0x9b, 0x74, 0x96, 0x26, 0x87, 0xf7, 0x2f, 0xaa, 0x0c, 0x33, 0x5c,
0x68, 0x25, 0xfc, 0x98, 0x7a, 0x7c, 0x50, 0x42, 0x4f, 0xcd, 0xef, 0x48, 0x3d, 0x49, 0x70, 0x38,
0x8c, 0xfc, 0x17, 0x81, 0x22, 0xa1, 0x21, 0xa8, 0x57, 0xb0, 0x78, 0xba, 0x5a, 0x5e, 0x96, 0xa8,
0x24, 0x3e, 0xe9, 0x87, 0xf0, 0xb2, 0x55, 0xbf, 0x37, 0x11, 0xf6, 0x36, 0x73, 0xfc, 0x92, 0x05
};

BYTE bAESSessionKey[16] =
{
0x19, 0x20, 0xbb, 0x9a, 0xa5, 0x60, 0x19, 0xb9, 0xf8, 0xdf, 0x00, 0xe2, 0x18, 0x91, 0x10, 0xb1
};
BYTE bAESSessionIV[16] =
{
0x4f, 0xce, 0xa3, 0x1d, 0xce, 0xa5, 0xca, 0xe1, 0x72, 0xfb, 0x64, 0xed, 0x1a, 0x1e, 0xe0, 0xbc
};
#define RSA_KEY_BITS_QUANTITY 512


#define lenCryptBuff (WORD)(512*1.5)
typedef BYTE tbCryptBuff[lenCryptBuff];

void PrintArray(const CHAR* name, PBYTE pData, WORD wLength)
{
	printf("BYTE %s[%d] =\n{\n", name, wLength);
	for (int i = 0; i < wLength; i++)
	{
		printf("0x%02x", pData[i]);
		if (i + 1 != wLength)
		{
			printf(", ");
		}
		if ((i + 1) % 16 == 0)
			printf("\n");
	}
	printf("};\n");
}

void PrintRSAKey(CBIOS_RSA_KEY* key)
{
	PrintArray("bRsaKeyModulus", key->ubModulus, key->wModulusLen);
	PrintArray("bRsaKeyExponent", key->ubPrivExp, key->wPrivExpLen);
}

DWORD GenerateKeys(INT32 iBoxIndex)
{
	DWORD dwRet, dwRet1;
	CBIOS_RSA_KEY keyPair, publicKey;
	CBIOS_RSA_KEY_INFO keyInfo;

	DWORD dwRSARandPoolSize;
	BYTE* pRSARandPool;

	DWORD dwAESRandPoolSize;
	BYTE* pAESRandPool;

	dwRSARandPoolSize = CBIOS_RAND_POOL_SIZE(RSA_KEY_BITS_QUANTITY);
	pRSARandPool = (BYTE*)malloc(dwRSARandPoolSize);

	printf(":: CBIOS_OpenByIndex\t");
	dwRet = CBIOS_OpenByIndex(iBoxIndex, gl_wTestAppId);
	PrintErrorCode(dwRet);

	if (dwRet == 0)
	{
		printf(":: CBIOS_GetHWRand\t");
		dwRet = CBIOS_GetHWRand(dwRSARandPoolSize, pRSARandPool);
		PrintErrorCode(dwRet);

		keyPair.dwStructSize = publicKey.dwStructSize = sizeof(CBIOS_RSA_KEY);
		keyPair.wModulusLen = keyPair.wPrivExpLen = RSA_KEY_BITS_QUANTITY;
		printf(":: CBIOS_GenerateKeyPairRSAEx\t");
		dwRet = CBIOS_GenerateKeyPairRSAEx(&keyPair, &publicKey, RSA_KEY_BITS_QUANTITY, pRSARandPool);
		PrintErrorCode(dwRet);
		free(pRSARandPool);


		dwAESRandPoolSize = CBIOS_AES_IV_LEN + CBIOS_AES_KEY_LEN;
		pAESRandPool = (BYTE*)malloc(dwAESRandPoolSize);
		printf(":: CBIOS_GetHWRand (AES)\t");
		dwRet = CBIOS_GetHWRand(dwAESRandPoolSize, pAESRandPool);
		PrintErrorCode(dwRet);
		
		memcpy(bAESSessionKey, pAESRandPool, CBIOS_AES_KEY_LEN);
		memcpy(bAESSessionIV, pAESRandPool + CBIOS_AES_KEY_LEN, CBIOS_AES_IV_LEN);
		free(pAESRandPool);

		printf(":: CBIOS_SetKeySession\t");
		dwRet = CBIOS_SetKeySession(bAESSessionKey);
		PrintErrorCode(dwRet);

		printf(":: CBIOS_SetKeySession\t");
		dwRet = CBIOS_SetIVSession(bAESSessionIV);
		PrintErrorCode(dwRet);

		printf(":: CBIOS_CryptSession (Public Modulus)\t");
		dwRet = CBIOS_CryptSession(publicKey.wModulusLen, publicKey.ubModulus);
		PrintErrorCode(dwRet);

		printf(":: CBIOS_CryptSession (Public Exponent)\t");
		CBIOS_CryptSession(publicKey.wModulusLen, publicKey.ubPrivExp);
		PrintErrorCode(dwRet);

		printf(":: CBIOS_CLose\t");
		dwRet1 = CBIOS_Close();
		PrintErrorCode(dwRet1);		

		if (dwRet) return dwRet;

		printf("\n");
		printf("Generated private RSA key:\n");
		printf("==========================\n");
		PrintRSAKey(&keyPair);
		printf("Encrypt UPW with this key and program to RAM3 of a CBU\n");

		rsaPrivateKey.dwStructSize = keyPair.dwStructSize;
		rsaPrivateKey.wModulusLen = keyPair.wModulusLen;
		rsaPrivateKey.wPrivExpLen = keyPair.wPrivExpLen;
		memcpy(rsaPrivateKey.ubModulus, keyPair.ubModulus, rsaPrivateKey.wModulusLen);
		memcpy(rsaPrivateKey.ubPrivExp, keyPair.ubPrivExp, rsaPrivateKey.wPrivExpLen);

		printf("\n");
		printf("Generated public RSA key(encrypted with CBU Fixed AES):\n");
		printf("=========================\n");
		PrintRSAKey(&publicKey);
		printf("Use this key in a protected application to decrypt password\n");

		printf("\nGenerated keypair(Pirvate RSA) will be used in this sample for CBU programming and in TestUPW function\n");

		rsaPublicKey.dwStructSize = publicKey.dwStructSize;
		rsaPublicKey.wModulusLen = publicKey.wModulusLen;
		rsaPublicKey.wPrivExpLen = publicKey.wPrivExpLen;
		memcpy(rsaPublicKey.ubModulus, publicKey.ubModulus, rsaPublicKey.wModulusLen);
		memcpy(rsaPublicKey.ubPrivExp, publicKey.ubPrivExp, rsaPublicKey.wPrivExpLen);

		printf("\nAES key used to encrypt RSA public key (store it in RAM3):\n");
		PrintArray("bAESSessionKey", bAESSessionKey, CBIOS_AES_KEY_LEN);
		PrintArray("bAESSessionIV", bAESSessionIV, CBIOS_AES_IV_LEN);
	}

	return dwRet;
}



DWORD ProgramUPW(INT32 iBoxIndex,  BYTE bUPW[UPW_LENGTH])
{
	CBIOS_BOX_INFO_ADV BoxInfo;
	DWORD dwRet;
	DWORD dwDoUPWId = TEOSDO_ENC_UPW_ID;
	DWORD dwDoUPWType = TEOSDO_CDO_MEMORY;
	DWORD dwDoUPWRAM = TEOSDO_ENC_UPW_RAM;
	DWORD dwDoUPWOffset = TEOSDO_ENC_UPW_OFFSET;	

	DWORD dwDoAESId = TEOSDO_SESSION_AES_ID;
	DWORD dwDoAESType = TEOSDO_CDO_MEMORY;
	DWORD dwDoAESRAM = TEOSDO_SESSION_AES_RAM;
	DWORD dwDoAESOffset = 0;

	BYTE* pRandomPool;
	DWORD dwEncBuf1Len;

	BYTE bAESKeyBuf[CBIOS_AES_KEY_LEN + CBIOS_AES_IV_LEN];

	tbCryptBuff bEncBuf;

	printf(":: CBIOS_OpenByIndex ");
	dwRet = CBIOS_OpenByIndex(iBoxIndex, gl_wTestAppId);
	PrintErrorCode(dwRet);
	if (dwRet == 0)
	{
		printf(":: TEOS_DoCreateReference - TEOSDO_CDO_MEMORY for encrypted UPW");
		dwRet = TEOS_DoCreateReference(dwDoUPWId, dwDoUPWType, dwDoUPWRAM, dwDoUPWOffset, 0);
		PrintErrorCode(dwRet);
		if (dwRet == 0)
		{
			printf(":: CBIOS_UPWLogin\t");
			dwRet = CBIOS_UPWLogin(bUPW);
			PrintErrorCode(dwRet);
			if (dwRet == 0)
			{
				dwEncBuf1Len = lenCryptBuff;
				printf(":: CBIOS_EncryptRSAEx (password)");
				dwRet = CBIOS_EncryptRSAEx(&rsaPrivateKey,
					CBIOS_RSA_PRIV_KEY | CBIOS_RSA_RSAREF_PADDING,
					bUPW,
					UPW_LENGTH,
					bEncBuf, 
					&dwEncBuf1Len);
				PrintErrorCode(dwRet);

				// Create Encrypted Memory DO (64 bytes) in RAM 3 for encrypted password
				printf(":: TEOS_DoSet (enc_password)");
				dwRet = cdo_set(dwDoUPWId, bEncBuf, dwEncBuf1Len);
				PrintErrorCode(dwRet);

				printf(":: TEOS_DoCreateReference - TEOSDO_CDO_MEMORY for AES");
				dwRet = TEOS_DoCreateReference(dwDoAESId, dwDoAESType, dwDoAESRAM, dwEncBuf1Len + TEOSDO_CDO_SIGNATURE_SIZE, 0);
				PrintErrorCode(dwRet);
				if (dwRet == 0)
				{
					memcpy(bAESKeyBuf, bAESSessionKey, CBIOS_AES_KEY_LEN);
					memcpy(bAESKeyBuf + CBIOS_AES_KEY_LEN, bAESSessionIV, CBIOS_AES_IV_LEN);

					printf(":: TEOS_DoSet (Session AES)");
					dwRet = cdo_set(dwDoAESId, bAESKeyBuf, CBIOS_AES_KEY_LEN + CBIOS_AES_IV_LEN);
					PrintErrorCode(dwRet);
				}

				printf(":: TEOS_DoDeleteReference");
				dwRet = TEOS_DoDeleteReference(dwDoAESId);
				PrintErrorCode(dwRet);

				printf(":: CBIOS_Logout\t\t");
				dwRet = CBIOS_Logout();
				PrintErrorCode(dwRet);
			}

			printf(":: TEOS_DoDeleteReference");
			dwRet = TEOS_DoDeleteReference(dwDoUPWId);
			PrintErrorCode(dwRet);
		}

		TEOS_DoClearReferences();

		printf(":: CBIOS_Close\t\t");
		dwRet = CBIOS_Close();
		PrintErrorCode(dwRet);
	}

	return 0;
}

DWORD ProtectedUPWLogin(CBIOS_RSA_KEY *pRsaKey)
{
	DWORD dwRet, dwRet1;
	DWORD dwDoUPWId = TEOSDO_ENC_UPW_ID;
	DWORD dwDoUPWType = TEOSDO_CDO_MEMORY;
	DWORD dwDoUPWRAM = TEOSDO_ENC_UPW_RAM;
	DWORD dwDoUPWOffset = TEOSDO_ENC_UPW_OFFSET;

	DWORD dwDoAESId = TEOSDO_SESSION_AES_ID;
	DWORD dwDoAESType = TEOSDO_CDO_MEMORY;
	DWORD dwDoAESRAM = TEOSDO_SESSION_AES_RAM;
	DWORD dwDoAESOffset = 0;

	DWORD dwEncUPWSize;
	DWORD dwAESSize;
	BYTE* pUPWData;
	BYTE* pAESData;

	CBIOS_RSA_KEY rsaKey;

	tbCryptBuff bDecBuf;
	DWORD dwDecBufLen;

	dwRet = TEOS_DoCreateReference(dwDoUPWId, dwDoUPWType, dwDoUPWRAM, dwDoUPWOffset, 0);
	if (dwRet == 0)
	{
		do
		{
			dwEncUPWSize = 0;
			dwRet = TEOS_DoGet(dwDoUPWId, NULL, &dwEncUPWSize, 0, NULL);
			if (dwRet != TEOS_ERR_BUFFER_TOO_SHORT)
			{
				break;
			};
			pUPWData = (BYTE*)malloc(dwEncUPWSize);
			dwRet = TEOS_DoGet(dwDoUPWId, pUPWData, &dwEncUPWSize, 0, NULL);
			PrintErrorCode(dwRet);
			if (dwRet == 0)
			{
				dwRet = TEOS_DoCreateReference(dwDoAESId, dwDoAESType, dwDoAESRAM, dwEncUPWSize + TEOSDO_CDO_SIGNATURE_SIZE, 0);


				dwRet = TEOS_DoGet(dwDoAESId, NULL, &dwAESSize, 0, NULL);
				if (dwRet != TEOS_ERR_BUFFER_TOO_SHORT) break;
				pAESData = (BYTE*)malloc(dwAESSize);
				dwRet = TEOS_DoGet(dwDoAESId, pAESData, &dwAESSize, 0, NULL);
				PrintErrorCode(dwRet);
				if (dwRet == 0)
				{
					dwRet = CBIOS_SetKeySession(pAESData);
					if (dwRet) break;

					dwRet = CBIOS_SetIVSession(pAESData + CBIOS_AES_KEY_LEN);
					if (dwRet) break;

					memcpy(pRsaKey, &rsaPublicKey, sizeof(rsaPublicKey));
					dwRet = CBIOS_CryptSession(pRsaKey->wModulusLen, pRsaKey->ubModulus);
					if (dwRet) break;
					dwRet = CBIOS_CryptSession(pRsaKey->wPrivExpLen, pRsaKey->ubPrivExp);
					if (dwRet) break;
					dwRet = CBIOS_DecryptRSAEx(pRsaKey, CBIOS_RSA_RSAREF_PADDING | CBIOS_RSA_PUBL_KEY, pUPWData, dwEncUPWSize, bDecBuf, &dwDecBufLen);
					if (dwRet) break;

					if (dwDecBufLen != UPW_LENGTH)
					{
						dwRet = CBIOS_ERR_CORRUPTED_CBIOS_DATA;
						break;
					}
					else if (!dwRet)
					{
						dwRet = CBIOS_UPWLogin(bDecBuf);
						if (dwRet) break;

					}
				}

				free(pAESData);
				memset(bDecBuf, 0, dwDecBufLen);
			}

			free(pUPWData);
		} while (0);
	}


	TEOS_DoClearReferences();


	return dwRet;
}

void TestUPW(INT32 iBoxIndex)
{
	DWORD dwRet;
	DWORD dwDoUPWId = TEOSDO_ENC_UPW_ID;
	DWORD dwDoUPWType = TEOSDO_CDO_MEMORY;
	DWORD dwDoUPWRAM = TEOSDO_ENC_UPW_RAM;
	DWORD dwDoUPWOffset = TEOSDO_ENC_UPW_OFFSET;

	DWORD dwDoAESId = TEOSDO_SESSION_AES_ID;
	DWORD dwDoAESType = TEOSDO_CDO_MEMORY;
	DWORD dwDoAESRAM = TEOSDO_SESSION_AES_RAM;
	DWORD dwDoAESOffset = 0;

	DWORD dwEncUPWSize;
	DWORD dwAESSize;
	BYTE* pUPWData;
	BYTE* pAESData;

	CBIOS_RSA_KEY rsaKey;

	tbCryptBuff bDecBuf;
	DWORD dwDecBufLen;

	printf(":: CBIOS_OpenByIndex ");
	dwRet = CBIOS_OpenByIndex(iBoxIndex, gl_wTestAppId);
	PrintErrorCode(dwRet);
	if (dwRet == 0)
	{
		printf(":: TEOS_DoCreateReference - TEOSDO_CDO_MEMORY");
		dwRet = TEOS_DoCreateReference(dwDoUPWId, dwDoUPWType, dwDoUPWRAM, dwDoUPWOffset, 0);
		PrintErrorCode(dwRet);
		if (dwRet == 0)
		{
			printf(":: TEOS_DoGet\t\t");
			dwEncUPWSize = 0;
			dwRet = TEOS_DoGet(dwDoUPWId, NULL, &dwEncUPWSize, 0, NULL);
			if (dwRet != TEOS_ERR_BUFFER_TOO_SHORT)
			{
				PrintErrorCode(dwRet);
				return;
			};
			pUPWData = (BYTE*)malloc(dwEncUPWSize);
			dwRet = TEOS_DoGet(dwDoUPWId, pUPWData, &dwEncUPWSize, 0, NULL);
			PrintErrorCode(dwRet);
			if (dwRet == 0)
			{

				printf(":: TEOS_DoCreateReference - TEOSDO_CDO_MEMORY");
				dwRet = TEOS_DoCreateReference(dwDoAESId, dwDoAESType, dwDoAESRAM, dwEncUPWSize + TEOSDO_CDO_SIGNATURE_SIZE, 0);
				PrintErrorCode(dwRet);

				dwRet = TEOS_DoGet(dwDoAESId, NULL, &dwAESSize, 0, NULL);
				if (dwRet != TEOS_ERR_BUFFER_TOO_SHORT)
				{
					PrintErrorCode(dwRet);
					return;
				};
				pAESData = (BYTE*)malloc(dwAESSize);
				dwRet = TEOS_DoGet(dwDoAESId, pAESData, &dwAESSize, 0, NULL);
				PrintErrorCode(dwRet);
				if (dwRet == 0)
				{
					printf(":: CBIOS_SetKeySession");
					dwRet = CBIOS_SetKeySession(pAESData);
					PrintErrorCode(dwRet);

					printf(":: CBIOS_SetKeySession");
					dwRet = CBIOS_SetIVSession(pAESData + CBIOS_AES_KEY_LEN);
					PrintErrorCode(dwRet);

					memcpy(&rsaKey, &rsaPublicKey, sizeof(rsaPublicKey));
					printf(":: CBIOS_CryptSession (RSA Modulus)");
					dwRet = CBIOS_CryptSession(rsaKey.wModulusLen, rsaKey.ubModulus);
					PrintErrorCode(dwRet);

					printf(":: CBIOS_CryptSession (RSA Exponent)");
					dwRet = CBIOS_CryptSession(rsaKey.wPrivExpLen, rsaKey.ubPrivExp);
					PrintErrorCode(dwRet);

					printf(":: CBIOS_DecryptRSAEx (password)");
					dwRet = CBIOS_DecryptRSAEx(&rsaKey, CBIOS_RSA_RSAREF_PADDING | CBIOS_RSA_PUBL_KEY, pUPWData, dwEncUPWSize, bDecBuf, &dwDecBufLen);
					PrintErrorCode(dwRet);

					if (dwDecBufLen != UPW_LENGTH)
					{
						printf("Password length mismatch\n");
					}
					else if (!dwRet)
					{
						printf(":: CBIOS_UPWLogin\t");
						dwRet = CBIOS_UPWLogin(bDecBuf);
						PrintErrorCode(dwRet);
						if (dwRet == 0)
						{
							printf(":: CBIOS_Logout\t");
							dwRet = CBIOS_Logout();
							PrintErrorCode(dwRet);
						}
					}
				}

				free(pAESData);
				memset(bDecBuf, 0, dwDecBufLen);
			}

			free(pUPWData);
		}

		TEOS_DoClearReferences();

		printf(":: CBIOS_Close\t\t");
		dwRet = CBIOS_Close();
		PrintErrorCode(dwRet);
	}

	return;
}


// This version of TestUPW uses obfy framework(https://github.com/fritzone/obfy)
// for an obfuscation. Use this function as an example when obfuscating your code
// Avoid showing messages from this functions as it might be a hook for
// a reverse-engeneering. If it can't be avoided then encrypt strings with
// AES or XOR it
#ifdef OBFUSCATE
DWORD OBF_TestUPW(INT32 iBoxIndex)
{
	DWORD dwRet;
	DWORD dwDoType = TEOSDO_CDO_MEMORY;
	DWORD dwDoId = TEOSDO_MEMORYOBJ_ID;
	DWORD dwDoRAM = TEOSDO_SAMPLE_RAM;
	DWORD dwDoOffset = TEOSDO_MEMORYOBJ_OFFSET;
	DWORD dwSize;
	BYTE* pData;

	tbCryptBuff bDecBuf;
	DWORD dwDecBufLen;

	OBF_BEGIN
		dwRet = CBIOS_OpenByIndex(iBoxIndex, V(gl_wTestAppId));

		IF(dwRet == 0)

			REPEAT
				dwRet = TEOS_DoCreateReference(dwDoId, dwDoType, dwDoRAM, dwDoOffset, 0);
				IF(dwRet == 0)
					dwSize = N(0);
					dwRet = TEOS_DoGet(dwDoId, NULL, &dwSize, N(0), NULL);
					IF(dwRet != TEOS_ERR_BUFFER_TOO_SHORT)
						TEOS_DoDeleteReference(dwDoId);
						BREAK;
					ENDIF
					pData = (BYTE*)malloc(dwSize);
					dwRet = TEOS_DoGet(dwDoId, pData, &dwSize, 0, NULL);
					IF(dwRet == N(0))
						dwRet = CBIOS_DecryptRSAEx(&rsaPublicKey, CBIOS_RSA_RSAREF_PADDING | CBIOS_RSA_PUBL_KEY, pData, dwSize, bDecBuf, &dwDecBufLen);
						IF(dwRet == N(0))
							dwRet = CBIOS_UPWLogin(bDecBuf);
							IF(dwRet == N(0))
								dwRet = CBIOS_Logout();
							ENDIF
						ENDIF
					ENDIF
					TEOS_DoDeleteReference(dwDoId);
					free(pData);
				ENDIF
			AS_LONG_AS(0)
			CBIOS_Close();
		ENDIF

	OBF_END

	return 0;
}
#endif

void InitRSAKeypairs()
{
	rsaPublicKey.dwStructSize = sizeof(CBIOS_RSA_KEY);
	rsaPublicKey.wModulusLen = sizeof(bRsaKeyModulus);
	rsaPublicKey.wPrivExpLen = sizeof(bRsaKeyExponent);
	memcpy(rsaPublicKey.ubModulus, bRsaKeyModulus, rsaPublicKey.wModulusLen);
	memcpy(rsaPublicKey.ubPrivExp, bRsaKeyExponent, rsaPublicKey.wPrivExpLen);

	rsaPrivateKey.dwStructSize = sizeof(CBIOS_RSA_KEY);
	rsaPrivateKey.wModulusLen = sizeof(bPrivRsaKeyModulus);
	rsaPrivateKey.wPrivExpLen = sizeof(bPrivRsaKeyExponent);
	memcpy(rsaPrivateKey.ubModulus, bPrivRsaKeyModulus, rsaPrivateKey.wModulusLen);
	memcpy(rsaPrivateKey.ubPrivExp, bPrivRsaKeyExponent, rsaPrivateKey.wPrivExpLen);
}

int main(int argc, char* argv[])
{
	DWORD dwRet;
	int n, i;
	BOOL bExit = FALSE;

	do
	{
		if (!SampleStart("Protection")) break;

		printf(":: CBIOS_Startup\t");
		dwRet = CBIOS_Startup();
		PrintErrorCode(dwRet);
		if (dwRet) break;

		InitRSAKeypairs();

		if (!ScanHardware(gl_wTestAppId, &gl_iBoxIndex))
		{
			bExit = TRUE;
			break;
		}

		while (1)
		{
			printf("\n====================================\n");
			printf("1. Generate RSA keypair\n");
			printf("2. Program cryptobox\n");
			printf("3. Test Protected UPW\n");
			printf("--------------------------------\n");
			printf("0. Exit\n");
			printf("\n");
			printf("Select opption (0..3) and press <ENTER>: \n");

			do
			{
				i = -1;
				scanf_s("%d", &i);
				if (i == -1)
				{
					CHAR ss[100];
					scanf_s("%s", ss);
				}
			} while ((i < 0) && (i > 2));

			if (i == 0)
				break;

			switch (i)
			{
			case 1:
				GenerateKeys(gl_iBoxIndex);
				break;
			case 2:
				ProgramUPW(gl_iBoxIndex, bUPW);
				break;
			case 3:
#ifdef OBFUSCATE
				OBF_TestUPW(gl_iBoxIndex);
#else
				TestUPW(gl_iBoxIndex);
#endif
			}
		}

	} while (0);


	return 0;
}