/*****************************************************************************
  Copyright:   : MARX(R) CryptoTech LP 2004, 2014
  Project      : Smarx tools for SmarxOS samples
  Created      : 15Jan2004                                    
  Tools        : MSVC++ 6.0 SP5 / Builder 5  and higher
  Description  : 
******************************************************************************
  VER         DATE        WHO   DETAIL                        
******************************************************************************
  1.00        15Jan2004   VS    Initial version
  2.00        07Feb2014   VG    Sample revised, teos_error.c renamed to smarx_utils.c
*****************************************************************************/
#include "smarx_utils.h"
#include <stdio.h>

#ifdef WIN32
#include <conio.h>
#else  
#include <unistd.h>
int getch(void)
{
 int i, j;
 i = getchar();
 while (j != 10)
 j = getchar();
 return i;
}
#endif // WIN32

#include "../include/cbios.h"
#ifndef _NO_TEOSDO_
#include "../include/teosdo.h"
#endif

#define BUFF_SIZE_100 100
#define BUFF_SIZE_200 200
#define BUFF_SIZE_512 512

bool gl_bNetworkMode = false;

// ---------------------------------------------------------------------------
const CHAR * GetModelName(DWORD dwModel)
{
	switch(dwModel)
	{
	case CBIOS_CB_MODEL_VERSA:	return STR_CB_VERSA;
	case CBIOS_CB_MODEL_XS:		return STR_CB_XS;
	case CBIOS_CB_MODEL_XL:		return STR_CB_XL;
	case CBIOS_CT_MODEL_XS:		return STR_CT_XS;
	case CBIOS_CT_MODEL_XL:		return STR_CT_XL; 
	default: return STR_CB_UNKNOWN;
	}
}


const CHAR * GetBoxTypeName (WORD wBoxType)
{
	switch(wBoxType)
	{
	case CBIOS_BOX_CBU:		return STR_CBU;
	case CBIOS_BOX_CBU2:	return STR_CBU2;
	default: return STR_CBU_UNKNOWN;
	}
}
// -----------------------------------------------------------------------
BOOL _ReqSmarxOSBox(void)
{
	char c;
	PrintError("SmarxOS(R) Hardware WAS NOT FOUND");
	printf("\nPlease, attach (demo) SmarxOS(R) CRYPTO-BOX and press any key...(Esc,0 - Exit)\n\n");
	c = _getch();
	if (c=='0' || c==27) return FALSE;
	return TRUE;
}
BOOL ScanHardware(WORD wAppId, INT32 *iBoxIndex)
{
	DWORD dwRet, a;
	INT32 i, iNumBoxes, iNumSmarxOSBoxes=0;
	BOOL fl_DemoSmarxOSBoxFound=FALSE;
	
	CBIOS_BOX_INFO_ADV BoxInfoAdv;
	CBIOS_BOX_INFO BoxInfo;
	BOOL flBoxInfoAdv = TRUE;
	CBIOS_APP_INFO AppInfo;
	char szLabel[CBIOS_LABEL_LEN+1] = {0};

	do
	{
		printf(":: CBIOS_ScanBoxes\t");
		iNumBoxes = CBIOS_ScanBoxes();
		if (iNumBoxes == 0)
		{
			if (!_ReqSmarxOSBox()) return FALSE;
			continue;
		}
		else printf("FOUND %i ITEMS:\n", iNumBoxes);

		for (*iBoxIndex = i = 1; i<=iNumBoxes; i++)
		{
			printf("\n\t CRYPTO-BOX #%i :\n", i);
			printf(":: CBIOS_IsBoxLockedByOthersI (%li)", i);
			if (CBIOS_IsBoxLockedByOthersI(i))
			{
				PrintError("[already used!]");
				continue;
			}
			printf(" [NO]\n");
			printf(":: CBIOS_GetBoxInfoAdvI (%li)", i);
			BoxInfoAdv.dwStructSize = sizeof(BoxInfoAdv);
			dwRet = CBIOS_GetBoxInfoAdvI(i, &BoxInfoAdv);
			if (dwRet == CBIOS_ERR_WRONG_PARAM)
			{
				flBoxInfoAdv = FALSE;
				PrintErrorCode(dwRet);
				if (i==1){ PrintError("Warning! Network Server is old !");printf("\n");}
				printf(":: CBIOS_GetBoxInfoI (%li)", i);
				dwRet = CBIOS_GetBoxInfoI(i, &BoxInfo);
			}
			else flBoxInfoAdv = TRUE;
			PrintErrorCode(dwRet);
			if (dwRet != CBIOS_SUCCESS) continue;

			if (flBoxInfoAdv) memcpy(&BoxInfo, &BoxInfoAdv.dwBoxName, sizeof(CBIOS_BOX_INFO));

			printf("   TYPE: %s\n", GetBoxTypeName(BoxInfoAdv.wType));
			printf("   MODEL: %s, FIRMWARE: %i.%i, SIZE: %iK RAM,\n",
				GetModelName(BoxInfo.dwModel),
				BoxInfo.bHiVersion, BoxInfo.bLoVersion, BoxInfo.dwRAMLen / 1024);

			if (0 == BoxInfo.wCBIOS)
			{
				printf("   BoxName (S/N): 0x%08lX, FORMAT: ", BoxInfo.dwBoxName );
				if (flBoxInfoAdv && BoxInfoAdv.bMPI != 0)
					PrintError("Box is MPI formatted.");
				else PrintError("Not compatible with SmarxOS(R)");
				printf("\n");
				continue;
			}
			iNumSmarxOSBoxes++;
			if (1==iNumSmarxOSBoxes)
			{
				*iBoxIndex = i;
			}

			memcpy(szLabel,BoxInfo.bBoxLabel,CBIOS_LABEL_LEN);
			printf("   BoxName (S/N): 0x%08lX,  Label: '%s'\n", BoxInfo.dwBoxName, szLabel);
			if (flBoxInfoAdv) printf("   DeveloperID: 0x%X\n\n",BoxInfoAdv.dwDeveloperID);

			for (a=1; a<=BoxInfo.dwApp; a++)
			{
				dwRet = CBIOS_GetAppInfoI(i, a, &AppInfo);
				if (dwRet != CBIOS_SUCCESS)
				{
					printf(":: CBIOS_GetAppInfoI(%i, %i)\t", i, a);
					PrintErrorCode(dwRet);
					continue;
				}
				printf("   App %i, RAM %i/%i/%i\n", AppInfo.wAppID, AppInfo.wRAM1Size, AppInfo.wRAM2Size, AppInfo.wRAM3Size);
			}
		}
		printf("\n");
		if (iNumSmarxOSBoxes==0)
			if (!_ReqSmarxOSBox()) return FALSE;
	} while(iNumSmarxOSBoxes==0);

	if (iNumSmarxOSBoxes==1)
	{
		printf("CRYPTO-BOX #%i will be selected for all further tests\n", *iBoxIndex);
		printf("\n:: PRESS ANY KEY ::\n");
        _getch();
	}
	else
	do
	{
		char c[2] = {0};
		printf("Select SmarxOS CRYPTO-BOX for all further tests : (%s:#%i)",
			fl_DemoSmarxOSBoxFound?"suggested":"default", *iBoxIndex);
		c[0] = _getch();
		if (!isdigit(c[0]) || c[0]=='0') {printf("%i\n",*iBoxIndex);break;}
		printf("%c\n",c[0]);
		i = atoi(c);
		if (i<1 || i>iNumBoxes) continue;
		printf(":: CBIOS_GetBoxInfoI (%li)", i);
		dwRet = CBIOS_GetBoxInfoI(i, &BoxInfo);
		PrintErrorCode(dwRet);
		if (dwRet == CBIOS_SUCCESS)
		{
			if (!BoxInfo.wCBIOS)
			{
				PrintError("Not compatible with SmarxOS(R)");printf("\n");
				continue;
			}
			*iBoxIndex = i;
			break;
		}
	}while(1);
return TRUE;
}
// -----------------------------------------------------------------------
BOOL ScanAndConnect()
{
	CBIOS_SERVER_INFO ServerInfo;
	int i, n;
	DWORD dwRet;
	char c[2] = {0};

	printf(":: CBIOS_ScanNetwork(2000 ms)\n");
	n = CBIOS_ScanNetwork(2000);
	if (0 == n)
	{
		printf("Smarx OS(R) Network Servers not found");
		printf("\nIn case of firewalling: grant access and search again");
		return FALSE;
	}
	printf("\n Smarx OS(R) Network Server(s) found:\n\n");

	for (i=1; i<=n; i++)
	{
		//printf(":: CBIOS_GetServerInfo(%i)", i);
		dwRet = CBIOS_GetServerInfo(i, &ServerInfo);
		if (dwRet != CBIOS_SUCCESS)
		{
			printf(":: CBIOS_GetServerInfo(%i)", i);
			PrintErrorCode(dwRet);
		}
		printf("(%i) - %s:%i - ",
				i, ServerInfo.szServerIp, ServerInfo.wServerPort);
		if (ServerInfo.iBoxesNumber>0)
			printf("[ attached %i CRYPTO-BOX-es ]\n", ServerInfo.iBoxesNumber);
		else printf("[ No CRYPTO-BOX-es attached ]\n");
	}

	printf("\n");
	if (n>1)
	do
	{
		printf("Select Smarx OS(R) Network Server (1..%i): ", n);
		c[0] = _getch();
		printf("%c\n",c[0]);
		i = atoi(c);
		if ((i>=1)&&(i<=n))
		   break;
	} while(1);
	else i=1;

	dwRet = CBIOS_GetServerInfo(i, &ServerInfo);
	if (dwRet==CBIOS_SUCCESS)
	{
		printf(":: CBIOS_Connect (%s:%i)", ServerInfo.szServerIp, ServerInfo.wServerPort);
		dwRet = CBIOS_Connect(ServerInfo.szServerIp, ServerInfo.wServerPort);
		PrintErrorCode(dwRet);
	}
	return dwRet==CBIOS_SUCCESS;
}

BOOL SampleStart(const char *szApiName)
{
	int flagNet=1;
	char c[2] = {0};

	printf(":::: %s API TEST PROGRAM ::::\n", szApiName);
	printf("====================================\n");
	printf("1. - Local mode\n");
	printf("2. - Network mode\n");
	printf("--------------------------------\n");
	printf("0. - Exit\n\n");
	do
	{
        printf("Select option (0..2): ");
        c[0] = _getch();
        printf("%c\n",c[0]);
        flagNet = atoi(c);
        if ((flagNet>=0)&&(flagNet<=2))
          break;
    } while(1);
	if (!flagNet) return FALSE;
    
	if (flagNet==2)
	{
		gl_bNetworkMode = false;
	  printf(":::: %s NET API TEST PROGRAM ::::\n\n", szApiName);
	  return ScanAndConnect();
	}
	printf(":::: %s API TEST PROGRAM ::::\n\n", szApiName);
	return TRUE;
}
// ======================================================================
// -----------------------------------------------------------------------
void PrintColorText(const char* szText, WORD wAttributes)
{
#ifndef LINUX
  HANDLE hStdHandle;
  hStdHandle = GetStdHandle(STD_OUTPUT_HANDLE);
  SetConsoleTextAttribute(hStdHandle, wAttributes);
#endif // LINUX

  printf(szText);

#ifndef LINUX
  SetConsoleTextAttribute(hStdHandle, FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE);
#endif // LINUX
}
void PrintError(const char* szErrorText)
{
	CHAR szBuff[BUFF_SIZE_512];

	printf("[");
	if (szErrorText == NULL) strcpy_s(szBuff, BUFF_SIZE_512, "ERROR");
	else sprintf_s(szBuff, BUFF_SIZE_512, "ERROR: %s", szErrorText);

	PrintColorText(szBuff, FOREGROUND_RED);
	printf("]");

#ifndef LINUX
	Beep(200,30);
#endif // LINUX
}

void PrintOK(const char* szText)
{
	printf("[");
	PrintColorText("OK", FOREGROUND_GREEN | FOREGROUND_INTENSITY);
	if (szText) printf(" (%s)]", szText);
	else printf("]");
}
// -----------------------------------------------------------------------
static void _ErrorToText(DWORD dwError, char* szError)
{
#ifndef _NO_TEOSDO_
	TEOS_ErrorToText(dwError, szError);
#else
	CBIOS_ErrorToText(dwError, szError);
#endif
}
void PrintErrorCode(DWORD dwError)
{
	char szTemp[100];

	printf("\t");
	switch (dwError)
	{
	case 0:
		PrintOK(NULL);
		break;
	default:
		_ErrorToText(dwError, szTemp);
		PrintError(szTemp);
	}
	printf("\n");
}
// -----------------------------------------------------------------------
void PrintExpectedError(DWORD dwError, DWORD dwExpected)
{
  char szTemp[BUFF_SIZE_200], szErr1[BUFF_SIZE_100], szErr2[BUFF_SIZE_100];
  
  printf("\t");
  _ErrorToText(dwError, szErr1);
  if (dwError == dwExpected) PrintOK(szErr1);
  else 
  {
	  _ErrorToText(dwExpected, szErr2);
	  if (dwError) sprintf_s(szTemp, BUFF_SIZE_200, "%s (EXPECTED: %s)", szErr1, szErr2);
	  else sprintf_s(szTemp, BUFF_SIZE_200, "EXPECTED: %s", szErr2);
	  PrintError(szTemp); 
  }
  printf("\n");
}

void PrintExpectedErrorEx(DWORD dwError, DWORD *pdwExpected)
{
	if (pdwExpected) PrintExpectedError(dwError, *pdwExpected);
	else PrintErrorCode(dwError);
}