unit programSample;

interface

uses Windows, SysUtils, cbios, testglobals, testutil;

const
  TEOSDO_CDO_SIZE = 12;
  TEOSDO_TEST_SIZE_1 = 4;
  TEOSDO_TEST_SIZE_2 = 8;
  TEOSDO_CDO_TEST_SIZE_1 = TEOSDO_TEST_SIZE_1 + TEOSDO_CDO_SIZE;
  TEOSDO_CDO_TEST_SIZE_2 = TEOSDO_TEST_SIZE_2 + TEOSDO_CDO_SIZE;

  TEOSDO_TEST_RAM = 1;
  TEOSDO_TEST_RAM_MEMORY = 3;

  // EXPIRATION DATE
  TEOSDO_TEST_1_DOID = 1001;
  TEOSDO_TEST_1_OFFSET = 0;

  // MEMORY
  TEOSDO_TEST_MEMORY_DOID = 3001;
  TEOSDO_TEST_MEMORY_OFFSET = 0;

var
  BoxInfo: CBIOS_BOX_INFO_ADV;
  isCbuSc: Boolean;


  gl_iBoxIndex: integer = 1;
  gl_iTestAppId: integer = 999;
  IsCrypToken: Boolean = false;
  testarr: array of integer;

  bUPW: TPasswd = (Ord('d'), Ord('e'), Ord('m'), Ord('o'), $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00);

Const
  iv: PTByteArr16 = @IV_S;

procedure ProgramCBU();

implementation

// -----------------------------------------------------------------------
Function distr_cdo_create_signed_data(data: PBYTE;
  const dataSize, boxName: DWORD; var signedData: PCDO_SIGNED_DATA;
  var signDataSize: DWORD): DWORD;
Var
  rsaPrivKey: CBIOS_RSA_KEY;
  refData: DO_RSA_EX_DATA;
  rsaDistr: DO_KeyRef;
  pSignedData: PCDO_SIGNED_DATA;
  dwRet, len: DWORD;
Begin
  len := CDO_SIGNED_DATA_SIZE(dataSize);
  FillChar(refData, sizeof(refData), $00);
  with refData do
  begin
    dwKeyIndex := Cardinal(DO_RSA_EX_KEY_INDEX_NONE);
    dwBoxName := boxName
  end;
  FillChar(rsaDistr, sizeof(rsaDistr), $00);
  with rsaDistr do
  begin
    keyType := DO_KEY_REF_KEY_TYPE_EXTNL;
    keyDescriptor := @refData;
    keyRSA := @rsaPrivKey
  end;

  signedData := Nil;
  signDataSize := 0;
  GetMem(pSignedData, len);
  FillChar(pSignedData^, sizeof(pSignedData^), $00);

  with rsaPrivKey do
  begin
    dwStructSize := sizeof(rsaPrivKey);
    wPrivExpLen := CBIOS_INTERNAL_RSA_LEN;
    wModulusLen := CBIOS_INTERNAL_RSA_LEN;
    Move(CBIOS_DEMO_DISTR_RSA_KEY_MODULUS, ubModulus, CBIOS_INTERNAL_RSA_LEN);
    Move(CBIOS_DEMO_DISTR_RSA_KEY_EXPONENT, ubPrivExp, CBIOS_INTERNAL_RSA_LEN);
  end;
  pSignedData^.dwSignSize := sizeof(pSignedData^.signature);
  dwRet := TEOS_CalculateDigitalSignature(Nil, @rsaDistr,
    TEOSDO_SIGN_ALG_TYPE_BOXNAME, 0, data, dataSize, @pSignedData^.signature,
    @pSignedData^.dwSignSize, Nil);
  if dwRet <> 0 then
  begin
    FreeMem(pSignedData);
    Result := dwRet;
    Exit;
  end;
  signedData := pSignedData;
  signDataSize := len;
  pSignedData^.dwDataSize := dataSize;
  Move(data^, pSignedData^.data, dataSize);

  Result := 0;
End;

Function client_get_boxname(Var boxName: DWORD): DWORD;
Var
  BoxInfo: CBIOS_BOX_INFO;
  dwRet: DWORD;
Begin
  dwRet := CBIOS_GetBoxInfo(@BoxInfo);
  if dwRet <> 0 then
  begin
    Result := dwRet;
    Exit
  end;
  boxName := BoxInfo.dwBoxName;
  Result := 0;
End;

Function client_cdo_op(dwDoId, dwOperation: DWORD; signedData: PCDO_SIGNED_DATA;
  signDataSize, dwParameter: DWORD; bPass: PTPasswd): DWORD;
Var
  dwRet, len: DWORD;
  pData: PBYTE;
Begin
  len := signDataSize;
  pData := PBYTE(signedData);

  dwRet := TEOS_DoOperation(dwDoId, dwOperation, pData, len,
    dwParameter, bPass);

  Result := dwRet
End;

// -----------------------------------------------------------------------
Function _cdo_op(dwDoId, dwOperation: DWORD; pData: PBYTE;
  dwDataSize, dwParameter: DWORD; bPass: PTPasswd; pdwExpected: PDWORD): DWORD;
Var
  signedData: PCDO_SIGNED_DATA;
  boxName, signedDataSize, dwRet: DWORD;
Begin
  signedData := Nil;

  Repeat
    dwRet := client_get_boxname(boxName); // or get from DB
    if dwRet <> 0 then
      break;

    dwRet := distr_cdo_create_signed_data(pData, dwDataSize, boxName,
      signedData, signedDataSize);
    if dwRet <> 0 then
      break;

    dwRet := client_cdo_op(dwDoId, dwOperation, signedData, signedDataSize,
      dwParameter, bPass);
    FreeMem(signedData);
  Until True;
  PrintExpectedErrorEx(dwRet, pdwExpected);
  Result := dwRet;
End;

// -----------------------------------------------------------------------

function cdo_set(dwDoId: DWORD; pData: PBYTE; dwDataSize: DWORD): DWORD;
begin
  // SET DATA OBJECT
  cdo_set := _cdo_op(dwDoId, TEOSDO_SET, pData, dwDataSize, 0, Nil, Nil)
end;

function cdo_set_error_checking(dwDoId: DWORD; pData: PBYTE;
  dwDataSize, dwExpected: DWORD): DWORD;
begin
  // SET DATA OBJECT
  Result := _cdo_op(dwDoId, TEOSDO_SET, pData, dwDataSize, 0, Nil, @dwExpected)
end;

// -----------------------------------------------------------------------

function cdo_def_unlim_ex(dwDoId: DWORD): DWORD;
begin
  // UNLIMITED DATA OBJECT
  Write(':: TEOS_DoUnlimitedEx'#9);
  Result := _cdo_op(dwDoId, TEOSDO_UNLIMITED_EX, Nil, 0, 0, Nil, Nil);
end;

function cdo_inc(dwDoId: DWORD; pData: PBYTE; dwDataSize: DWORD): DWORD;
begin
  // SET DATA OBJECT
  Result := _cdo_op(dwDoId, TEOSDO_INC, pData, dwDataSize, 0, Nil, Nil);
end;

function cdo_def_inc(dwDoId: DWORD; sUnit: string): DWORD;
begin
  // INCREMENT DATA OBJECT
  Write(':: TEOS_DoInc (1 ' + sUnit + ')'#9);
  Result := _cdo_op(dwDoId, TEOSDO_INC, Nil, 0, 0, Nil, Nil);
end;

// -----------------------------------------------------------------------

function _do_def_dec(dwDoId: DWORD; sUnit: string; pdwExpected: PDWORD): DWORD;
var
  dwRet: DWORD;
begin
  // DECREMENT DATA OBJECT
  Write(':: TEOS_DoDec (1 ' + sUnit + ')');
  if Length(sUnit) < 4 then
    Write(#9);
  dwRet := TEOS_DoDec(dwDoId, Nil, 0, 0, Nil);
  PrintExpectedErrorEx(dwRet, pdwExpected);
  Result := dwRet
end;

function do_def_dec(dwDoId: DWORD; sUnit: string): DWORD;
begin
  Result := _do_def_dec(dwDoId, sUnit, Nil);
end;

function do_def_dec_error_checking(dwDoId: DWORD; sUnit: string;
  dwExpected: DWORD): DWORD;
begin
  Result := _do_def_dec(dwDoId, sUnit, @dwExpected);
end;

// -----------------------------------------------------------------------
function do_def_clear(dwDoId: DWORD): DWORD;
var
  dwRet: DWORD;
begin
  // CLEAR DATA OBJECT
  Write(':: TEOS_DoClear'#9#9);
  dwRet := TEOS_DoClear(dwDoId, Nil);
  PrintDOErrorCode(dwRet);
  Result := dwRet;
end;

// -----------------------------------------------------------------------
function _do_verify(dwDoId: DWORD; pdwData, pdwExpected: PDWORD): DWORD;
var
  dwRet, dwDataSize: DWORD;
begin
  if pdwData = Nil then
    dwDataSize := 0
  else
    dwDataSize := sizeof(DWORD);
  // VERIFY
  Write(':: TEOS_DoVerify');
  if pdwData <> Nil then
    Write(' (by ', pdwData^, ')')
  else
    Write(#9);
  dwRet := TEOS_DoVerify(dwDoId, PBYTE(pdwData), dwDataSize, 0, Nil);
  PrintExpectedErrorEx(dwRet, pdwExpected);
  Result := dwRet;
end;

function _do_def_verify(dwDoId: DWORD; pdwExpected: PDWORD): DWORD;
begin
  Result := _do_verify(dwDoId, Nil, pdwExpected);
end;

function do_verify(dwDoId, dwData: DWORD): DWORD;
begin
  Result := _do_verify(dwDoId, @dwData, Nil);
end;

function do_verify_error_checking(dwDoId, dwData, dwExpected: DWORD): DWORD;
begin
  Result := _do_verify(dwDoId, @dwData, @dwExpected);
end;

function do_def_verify(dwDoId: DWORD): DWORD;
begin
  Result := _do_def_verify(dwDoId, Nil);
end;

function do_def_verify_error_checking(dwDoId, dwExpected: DWORD): DWORD;
begin
  Result := _do_def_verify(dwDoId, @dwExpected);
end;

// -----------------------------------------------------------------------
Procedure PrintErrorCode(dwError: DWORD);
Begin
  PrintDOErrorCode(dwError);
End;

// -----------------------------------------------------------------------
Procedure PrintDataObjectValue(dwData: DWORD; szUnits: string);
Begin
  if (dwData = TEOSDO_UNLIMITED_VALUE) then
    WriteLn(#9#9#9#9'UNLIMITED')
  else
    WriteLn(#09#09#09#09, LongInt(dwData), ' ', szUnits, ' left');
End;

Procedure PrintDataObjectValueTime(dwSeconds: DWORD);
Const
  hoursecs = 60 * 60;
  daysecs = SECADAY;
Begin
  if dwSeconds = TEOSDO_UNLIMITED_VALUE then
    Exit;
  Write(#9#9#9#9);
  if dwSeconds div daysecs > 0 then
    Write(Format('%dd:', [dwSeconds div daysecs]));
  WriteLn(Format('%.2dh:%.2dm:%.2ds left',
    [(dwSeconds mod daysecs) div hoursecs, (dwSeconds mod hoursecs) div 60,
    dwSeconds mod 60]));
End;

Function _do_get_value_ex(dwDoId: DWORD; sUnits: string;
  pdwData, pdwExpected: PDWORD): DWORD;
Var
  dwRet, dwData, dwDataSize: DWORD;
Begin
  // GET DATA OBJECT
  Write(':: TEOS_DoGet');
  if pdwData = Nil then
    pdwData := @dwData;
  pdwData^ := 0;
  dwDataSize := sizeof(DWORD);
  dwRet := TEOS_DoGet(dwDoId, PBYTE(pdwData), @dwDataSize, 0, Nil);
  if dwRet <> 0 then
  begin
    Write(#9#9);
    PrintExpectedErrorEx(dwRet, pdwExpected);
  end
  else
    PrintDataObjectValue(pdwData^, sUnits);
  Result := dwRet
End;

Function _do_get_value(dwDoId: DWORD; sUnits: string;
  pdwExpected: PDWORD): DWORD;
Begin
  Result := _do_get_value_ex(dwDoId, sUnits, Nil, pdwExpected);
End;

function cbusc_crypt(iv: PTByteArr16; pData: PBYTE;
  dwDataSize: LongInt): LongInt;
begin
  Result := 0;
end;

function bintostr(const bin: array of byte): string;
const HexSymbols = '0123456789ABCDEF';
var i: integer;
begin
  SetLength(Result, 2*Length(bin));
  for i :=  0 to Length(bin)-1 do begin
    Result[1 + 2*i + 0] := HexSymbols[1 + bin[i] shr 4];
    Result[1 + 2*i + 1] := HexSymbols[1 + bin[i] and $0F];
  end;
end;

procedure ProgramCBU();
var
  dwDoId, dwDoType, dwDoRAM, dwDoOffset: DWORD;
  dwRet, dwDataSize: DWORD;
  bPassword: TPasswd;
  strDate: string;
  szTemp : AnsiString;
begin
  if True = ScanHardware(gl_iBoxIndex, IsCrypToken) then
  begin
    dwRet := CBIOS_OpenByIndex(gl_iBoxIndex, gl_iTestAppId);

    if (dwRet = 0) then
    begin
      Repeat
        BoxInfo.dwStructSize := sizeof(BoxInfo);
        dwRet := CBIOS_GetBoxInfoAdv(@BoxInfo);
        isCbuSc := CBIOS_BOX_CBU2 = BoxInfo.wType;

        dwDoId := TEOSDO_TEST_MEMORY_DOID;
        dwDoType := TEOSDO_CDO_MEMORY;
        dwDoRAM := TEOSDO_TEST_RAM_MEMORY;
        dwDoOffset := TEOSDO_TEST_MEMORY_OFFSET;

        WriteLn('Program encrypted UPW to a CBU RAM3 zone');
        WriteLn('Password: ', bintostr(bUpw));
        dwRet := TEOS_DoCreateReference(dwDoId, dwDoType, dwDoRAM,
          dwDoOffset, 0);
        if (dwRet = 0) then
        begin
          Move(bUpw, bPassword, sizeof(bPassword));
          dwRet := CBIOS_CryptFixed(sizeof(bUpw), @bPassword);
          Write(':: CBIOS_CryptFixed');
          PrintErrorCode(dwRet);
          if (dwRet = 0) then
          begin
            dwRet := cdo_set(dwDoId, @bPassword, sizeof(bPassword));
            Write(':: cdo_set TEOSDO_CDO_MEMORY ');
            PrintErrorCode(dwRet);

            TEOS_DoDeleteReference(dwDoId);
          end;
        end
        else break;

         WriteLn('Program encrypted expiration date to a CBU RAM1 zone');
        dwRet := CBIOS_UPWLogin(@bUpw);
        if dwRet = 0 then
        begin
          // Program Expiration date
          dwDoId := TEOSDO_TEST_1_DOID;
          dwDoType := TEOSDO_CDO_EXPIRATION_DATE_AND_TIME;
          dwDoRAM := TEOSDO_TEST_RAM;
          dwDoOffset := 0;

          dwRet := TEOS_DoCreateReference(dwDoId, dwDoType, dwDoRAM,
            dwDoOffset, 0);

          if (dwRet = 0) then
          begin
            szTemp := '01 AUG 2023';
            dwDataSize:=Length(szTemp);
            dwRet := cdo_set(dwDoId, @szTemp[1], dwDataSize);
            Write(':: cdo_set TEOSDO_CDO_EXPIRATION_DATE_AND_TIME ');
            PrintErrorCode(dwRet);

            TEOS_DoDeleteReference(dwDoId);
          end;
          CBIOS_Logout();
        end
        else break;

      Until True;

    end;




    CBIOS_Logout();
    CBIOS_Close();


  end;

end;

//procedure Test();
//begin
//  ProgramUPW;
//end;

end.
