{*****************************************************************************
Copyright:   : MARX CryptoTech LP
Project      : TEOSDO: Smarx OS(R) DATA OBJECTS API
Created      : 01Jul2003
Tools        : Delphi 5-XE
Description  : Smarx OS(R) DATA OBJECTS API test
******************************************************************************
VER         DATE        WHO   DETAIL
******************************************************************************
1.00        15Apr2022   OK    Initial version
*****************************************************************************}

Unit sample;

Interface

Uses SysUtils, System.Classes, FMX.Dialogs,
{$IF DEFINED(MACOS)}
 Macapi.Foundation,
{$ELSE}
 Windows,
{$ENDIF}

testglobals, testutil, cbios, Main;

Const
  TEOSDO_CDO_SIZE        =     12;
  TEOSDO_TEST_SIZE_1     =      4;
  TEOSDO_TEST_SIZE_2     =      8;
  TEOSDO_CDO_TEST_SIZE_1 =  TEOSDO_TEST_SIZE_1+TEOSDO_CDO_SIZE;
  TEOSDO_CDO_TEST_SIZE_2 =  TEOSDO_TEST_SIZE_2+TEOSDO_CDO_SIZE;

  TEOSDO_TEST_RAM        =       1;
  TEOSDO_TEST_RAM_MEMORY = 3;

  // ENCRYPTED PASSWORD
  TEOSDO_TEST_MEMORY_DOID = 3001;
  TEOSDO_TEST_MEMORY_OFFSET = 0;


  // EXPIRATION DATE
  TEOSDO_TEST_1_DOID     =    1001;
  TEOSDO_TEST_1_OFFSET   =       0;
 
{$ifdef WIN64}
	SZ_WIN    = 'Win64';
{$else}
	SZ_WIN    = 'Win32';
{$endif}
	SZ_WIN_MARXPATH = '\MARX\API\' + SZ_WIN + '\DO\VC';

Var gl_iBoxIndex : Integer = 1;
  test: array [0..35] of Byte;//for upw

    bUPW: TPasswd = ($00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00, $00);

  //****
  gl_ServerInfo : CBIOS_SERVER_INFO;
  //****

//****
Procedure SetNotifications(flag:Boolean);
Function  CBUTest1() : Boolean;
//****

Implementation

uses testThread;

{$IFDEF CONDITIONALEXPRESSIONS}
{$IF CompilerVersion >= 16}
    {$DEFINE Def_ShlwApi}
    {$DEFINE Def_SHCreateDirectory}
{$IFEND}
{$ENDIF}



{$IFNDEF Def_ShlwApi}
function PathAppend(pszPath,pszMore:PChar):LongBool;
         stdcall; external 'shlwapi.dll' name 'PathAppendA'
{$ENDIF}

{$IFNDEF Def_SHCreateDirectory}
function SHCreateDirectoryExA(hwnd: HWND; pszPath: LPCSTR; psa: PSecurityAttributes): integer;
         stdcall; external 'shell32.dll' name 'SHCreateDirectoryExA';
function SHCreateDirectory(hwnd: HWND; pszPath: LPCSTR): integer;
begin
  Result := SHCreateDirectoryExA(hwnd, pszPath, Nil);
end;
{$ENDIF}

// -----------------------------------------------------------------------
Function distr_cdo_create_signed_data(data : PBYTE; const dataSize, boxName : DWORD;
								   var signedData : PCDO_SIGNED_DATA; var signDataSize : DWORD) : DWORD;
Var
  rsaPrivKey : CBIOS_RSA_KEY;
  refData    : DO_RSA_EX_DATA;
  rsaDistr   : DO_KeyRef;
  pSignedData : PCDO_SIGNED_DATA;
  dwRet, len : DWORD;
Begin
  len := CDO_SIGNED_DATA_SIZE(dataSize);
  FillChar(refData, sizeof(refData), $00);
  with refData do
  begin
    dwKeyIndex := Cardinal(DO_RSA_EX_KEY_INDEX_NONE);
    dwBoxName := boxName
  end;
  FillChar(rsaDistr, sizeof(rsaDistr), $00);
  with rsaDistr do
  begin
    keyType := DO_KEY_REF_KEY_TYPE_EXTNL;
    keyDescriptor := @refData;
    keyRSA := @rsaPrivKey
  end;

  signedData := Nil;
  signDataSize := 0;
  GetMem(pSignedData, len);
  FillChar(pSignedData^, sizeof(pSignedData^), $00);

  with rsaPrivKey do
  begin
    dwStructSize := sizeof(rsaPrivKey);
    wPrivExpLen := CBIOS_INTERNAL_RSA_LEN;
    wModulusLen := CBIOS_INTERNAL_RSA_LEN;
    Move(CBIOS_DEMO_DISTR_RSA_KEY_MODULUS,  ubModulus,  CBIOS_INTERNAL_RSA_LEN);
    Move(CBIOS_DEMO_DISTR_RSA_KEY_EXPONENT,  ubPrivExp,  CBIOS_INTERNAL_RSA_LEN);
  end;
  pSignedData^.dwSignSize := sizeof(pSignedData^.signature);
  dwRet := TEOS_CalculateDigitalSignature(Nil, @rsaDistr, TEOSDO_SIGN_ALG_TYPE_BOXNAME, 0, data, dataSize, @pSignedData^.signature, @pSignedData^.dwSignSize, Nil);
  if dwRet <> 0 then
  begin
    FreeMem(pSignedData);
    Result := dwRet;
    Exit;
  end;
  signedData := pSignedData;
  signDataSize := len;
  pSignedData^.dwDataSize := dataSize;
  Move(data^, pSignedData^.data, dataSize);

  Result := 0;
End;

Function client_get_boxname(Var boxName : DWORD) : DWORD;
Var
  boxInfo : CBIOS_BOX_INFO;
  dwRet   : DWORD;
Begin
  dwRet := CBIOS_GetBoxInfo(@boxInfo);
  if dwRet<>0 then
  begin
    Result := dwRet;
    Exit
  end;
  boxName := boxInfo.dwBoxName;
  Result := 0;
End;

Function client_cdo_op(dwDoID, dwOperation : DWORD; signedData : PCDO_SIGNED_DATA; signDataSize, dwParameter : DWORD; bPass : PTPasswd) : DWORD;
Var
  dwRet, len : DWORD;
  pData : PBYTE;
Begin
	len := signDataSize;
  pData := PBYTE(signedData);

	dwRet := TEOS_DoOperation(dwDoID, dwOperation, pData, len, dwParameter, bPass);

	Result := dwRet
End;

// -----------------------------------------------------------------------
Function _cdo_op(dwDoID, dwOperation : DWORD; pData : PBYTE; dwDataSize, dwParameter : DWORD; bPass : PTPasswd; pdwExpected : PDWORD) : DWORD;
Var
  signedData : PCDO_SIGNED_DATA;
  boxName, signedDataSize, dwRet : DWORD;
Begin
  signedData:=Nil;

	Repeat
		dwRet := client_get_boxname(boxName);//or get from DB
		if dwRet<>0 then break;

		dwRet := distr_cdo_create_signed_data(pData, dwDataSize, boxName, signedData, signedDataSize);
		if dwRet<>0 then break;

		dwRet := client_cdo_op(dwDoID, dwOperation, signedData, signedDataSize, dwParameter, bPass);
		FreeMem(signedData);
	Until True;
	PrintExpectedErrorEx(dwRet, pdwExpected);
	Result := dwRet;
End;

// -----------------------------------------------------------------------

function cdo_set(dwDoID:DWORD; pData:PBYTE; dwDataSize:DWORD) : DWORD;
begin
	// SET DATA OBJECT
	cdo_set := _cdo_op(dwDoID, TEOSDO_SET, pData, dwDataSize, 0, Nil, Nil)
end;

// -----------------------------------------------------------------------
procedure teosdo_get_memory(dwDoId: DWORD; dwOffset: DWORD; dwSize: DWORD);
var  a, dwRet: DWORD;
     pData, p: PBYTE;
     b: integer;
begin

  GetMem(pData, dwSize);
  dwRet:= TEOS_DoGet(dwDoId, pData, @dwSize, dwOffset, nil);
  if (dwRet = 0) then
  begin
    p:= pData;
    for a:= 0 to dwSize-1 do
     begin
       b:= p^;
       write(IntToHex(b,2) + ' ');
       Inc(p);
     end;
  end;
  FreeMem(pData);
end;




function cbusc_crypt(iv : PTByteArr16; pData: PByte; dwDataSize:Longint) : Longint;
begin
	Result := 0;
	//Please see CBU2_Sample project (test_CBU2.pas)

	//write(':: CBIOS_CBU2_CryptAES'#9);
	//dwRet := CBIOS_CBU2_CryptAES(0, CBIOS_AES_OFB, iv, pData, pData, dwDataSize);
	//PrintErrorCode(dwRet);
	//Result := dwRet;
end;

procedure teosdo_get_cdo_memory(dwDoId: DWORD; isString, isCbuSc: Boolean; iv: PTByteArr16);
var  a, dwRet, dwSize: DWORD;
     pData, p: PBYTE;
     b: integer;
     s : AnsiString;
begin
  dwSize := 0;

  dwRet:= TEOS_DoGet(dwDoId, nil, @dwSize, 0, nil);
  if dwRet <> TEOS_ERR_BUFFER_TOO_SHORT then begin
	  Exit;
  end;

  GetMem(pData, dwSize);
  dwRet:= TEOS_DoGet(dwDoId, pData, @dwSize, 0, nil);

  if (dwRet = 0) then
  begin
    if isCbuSc then cbusc_crypt(iv, pData, dwSize);
    p:= pData;
    if isString then Write(PString(pData)^)
    else for a:= 0 to dwSize-1 do
     begin
       b:= p^;
       Inc(p);
     end;
  end;
  FreeMem(pData);
end;


procedure teosdo_get_cdo_memory2(dwDoId: DWORD; isString, isCbuSc: Boolean; iv: PTByteArr16; pResData: Pointer; dwResSize: DWORD);
var  a, dwRet, dwSize: DWORD;
     pData, p: PBYTE;
     b: integer;
     s : AnsiString;
begin
  dwSize := 0;

  dwRet:= TEOS_DoGet(dwDoId, nil, @dwSize, 0, nil);
  if dwRet <> TEOS_ERR_BUFFER_TOO_SHORT then begin
	  Exit;
  end;

  if (dwSize < dwResSize) then EXIT;

  GetMem(pData, dwSize);
  dwRet:= TEOS_DoGet(dwDoId, pData, @dwSize, 0, nil);

  if (dwRet = 0) then
  begin
    if isCbuSc then cbusc_crypt(iv, pData, dwSize);
    CopyMemory(pResData, pData, dwSize);
  end;
  FreeMem(pData);
end;



Var dwNotifyCount : LongInt = 0;
function BoxNotify(NotificationData:CBIOS_NOTIFY_DATA) : LongInt;stdcall;
Var
  dwRet : LongInt;
  dwRetOpen, dwRetIn, dwRetOut, dwRetClose : LongInt;
  szd: array [0..35] of Byte;
  co: TTestThread;
begin
  //WriteLn;
  case (NotificationData.dwNotificationType) of
      CBIOS_BOX_REMOVED:
        begin

          co := TTestThread.Create(true);
          co.FreeOnTerminate := true;
          co.Start;
        end;
      else begin
        //WriteLn(#9'...unknown notification received');
        co := TTestThread.Create(true);
        co.FreeOnTerminate := true;
        co.Start;
      end;
	end;//case

	Inc(dwNotifyCount);
	BoxNotify := 0;
end;

procedure SetNotifications(flag:Boolean);
Var
  dwRet : LongInt;
  Param : NativeUInt;
begin
  dwNotifyCount := 0;
  if (flag = true) then begin
    Param := 1;
    dwRet := CBIOS_RegisterNotificationCallback(@BoxNotify, Param);
  end else begin
    dwRet := CBIOS_UnregisterNotificationCallback(@BoxNotify);
  end;
end;


Function  CBUTest1() : Boolean;
var
  dwRet: DWORD;
  res: BOOL;
  k: integer;

  dwDoId, dwDoType, dwDoRAM, dwDoOffset: DWORD;
  BoxInfo: CBIOS_BOX_INFO_ADV;
  isCbuSc : BOOL;
  pData: PBYTE;
  szTemp : AnsiString;
  dwDataSize: DWORD;
Const
    iv : PTByteArr16 = @IV_S;

begin
  res := false;

  dwRet := 1;

  Randomize;

  if TRUE = ScanHardware(gl_iBoxIndex,  IsCrypToken) then begin
    dwRet := CBIOS_OpenByIndex(gl_iBoxIndex, gl_iTestAppId );

    if (dwRet = 0) then begin
      BoxInfo.dwStructSize := sizeof(BoxInfo);
      dwRet := CBIOS_GetBoxInfoAdv(@BoxInfo);
      isCbuSc := CBIOS_BOX_CBU2 = boxInfo.wType;

      dwDoId := TEOSDO_TEST_MEMORY_DOID;
      dwDoType := TEOSDO_CDO_MEMORY;
      dwDoRAM := TEOSDO_TEST_RAM_MEMORY;
      dwDoOffset := TEOSDO_TEST_MEMORY_OFFSET;

      dwRet:= TEOS_DoCreateReference(dwDoId, dwDoType, dwDoRAM, dwDoOffset, 0);
      if (dwRet = 0) then begin
        teosdo_get_cdo_memory2(dwDoId, False, isCbuSc, nil, @bUPW, 16);
        TEOS_DoDeleteReference(dwDoId);
        dwRet := CBIOS_CryptFixed(sizeof(bUPW), @bUPW);
        if dwRet = 0 then
        begin
          dwRet := CBIOS_UPWLogin(@bUpw);

          // Verify Expiration Date
          if dwRet = 0 then
          begin
            SetLength(szTemp,128);
            dwDataSize:=Length(szTemp);

            dwDoId := TEOSDO_TEST_1_DOID;
            dwDoType := TEOSDO_CDO_EXPIRATION_DATE_AND_TIME;
            dwDoRAM := TEOSDO_TEST_RAM;
            dwDoOffset := 0;
            dwRet:= TEOS_DoCreateReference(dwDoId, dwDoType, dwDoRAM, dwDoOffset, 0);

            dwRet:=TEOS_DoGet(dwDoId, @szTemp[1], @dwDataSize, TEOSDO_DATE_AS_STRING, nil);
            if dwRet = 0 then
            begin
              SetLength(szTemp,128);
              dwDataSize:=Length(szTemp);
              dwRet:=TEOS_DoVerify(dwDoId, @szTemp[1], dwDataSize, 0, nil);
              if dwRet <> 0 then
                res := false;
              TEOS_DoDeleteReference(dwDoId);

            end

          end;

        end;

      end;
    end;

    CBIOS_Logout();
    CBIOS_Close();

    if (dwRet = 0) then begin
      res := true;
    end;


  end;

  Result := res;
end;


End.
